#------------------------------------------------------------------------------
# This file is part of the OpenStructure project <www.openstructure.org>
#
# Copyright (C) 2008-2020 by the OpenStructure authors
#
# This library is free software; you can redistribute it and/or modify it under
# the terms of the GNU Lesser General Public License as published by the Free
# Software Foundation; either version 3.0 of the License, or (at your option)
# any later version.
# This library is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
# FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public License for more
# details.
#
# You should have received a copy of the GNU Lesser General Public License
# along with this library; if not, write to the Free Software Foundation, Inc.,
# 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
#------------------------------------------------------------------------------
"""
Wrapper for the CAD score.

References:

Olechnovic K, Kulberkyte E, Venclovas C., CAD-score: A new contact area 
difference-based function for evaluation of protein structural models
Proteins. 2012 Aug 30. [Epub ahead of print]

Authors: Valerio Mariani, Alessandro Barbato
"""

import subprocess, os, tempfile, platform, re
from ost import settings, io, mol

def _SetupFiles(model,reference):
  # create temporary directory
  tmp_dir_name=tempfile.mkdtemp()
  dia = 'PDB'
  for chain in model.chains:
    if chain.name==" ":
      raise RuntimeError("One of the chains in the model has no name. Cannot "
                         "calculate CAD score")
    if len(chain.name) > 1:
      dia = 'CHARMM'
      break;
    for res in chain.residues:
      if len(res.name) > 3:
        dia = 'CHARMM'
        break;
  io.SavePDB(model, os.path.join(tmp_dir_name, 'model.pdb'), dialect=dia)
  dia = 'PDB'
  for chain in reference.chains:
    if chain.name==" ":
      raise RuntimeError("One of the chains in the reference has no name. "
                         "Cannot calculate CAD score")
    if len(chain.name) > 1:
      dia = 'CHARMM'
      break;
    for res in chain.residues:
      if len(res.name) > 3:
        dia = 'CHARMM'
        break;
  io.SavePDB(reference, os.path.join(tmp_dir_name, 'reference.pdb'),dialect=dia)
  return tmp_dir_name

def _CleanupFiles(dir_name):
  import shutil
  shutil.rmtree(dir_name)

class CADResult:
  """
  Holds the result of running CAD
  
  .. attribute:: globalAA
    
    The global CAD's atom-atom (AA) score
  
  .. attribute:: localAA
  
    Dictionary containing local CAD's atom-atom (AA) scores. 
    
    :type: dictionary (key: tuple(chain, resnum) (e.g.: 
           ("A", ost.mol.ResNum(24)), value: CAD local AA score 
           (see CAD Documentation online)
  """
  def __init__(self, globalAA, localAA):    
    self.globalAA=globalAA
    self.localAA=localAA    

def _ParseCADGlobal(lines):
  header = lines[0].split()
  aa_idx = header.index("AA")
  aa_score=float(lines[1].split()[aa_idx])
  return aa_score

def _ParseCADLocal(lines):
  local_scores_idx = None
  for line_idx in range(len(lines)):
    if "local_scores" in lines[line_idx]:
      local_scores_idx = line_idx
      break
  if local_scores_idx == None:
    raise RuntimeError("Failed to parse local cadscores")
  local_aa_dict={}
  for line_idx in range(local_scores_idx+2, len(lines)):
    items=lines[line_idx].split()
    local_aa = float(items[2])
    if local_aa < 0.0:
      continue # invalid CAD score
    key = (items[0], mol.ResNum(int(items[1])))
    local_aa_dict[key] = local_aa
    
  return local_aa_dict

def _ParseVoronotaGlobal(lines):
  return float(lines[0].split()[4])

def _ParseVoronotaLocal(lines):
  local_aa_dict={}
  chain_name_regex = r'c\<\D+\>'
  resnum_regex = r'r\<\d+\>'
  insertion_code_regex = r'i\<\D\>'
  for line in lines:
    local_aa = float(line.split()[-1])
    if local_aa < 0.0:
      continue # invalid CAD score
    chain_data = re.findall(chain_name_regex, line)
    resnum_data = re.findall(resnum_regex, line)
    insertion_code_data = re.findall(insertion_code_regex, line)
    resnum = None
    if len(insertion_code_data) == 0:
      resnum = mol.ResNum(int(resnum_data[0][1:].strip('><')))
    else:
      resnum = mol.ResNum(int(resnum_data[0][1:].strip('><')), 
                          insertion_code_data[0][1:].strip('><'))
    key = (chain_data[0][1:].strip('><'), resnum)
    local_aa_dict[key] = local_aa
  return local_aa_dict

def _RunCAD(tmp_dir, mode, cad_bin_path, old_regime):

  model_filename=os.path.join(tmp_dir, 'model.pdb')
  reference_filename=os.path.join(tmp_dir, 'reference.pdb')
  globalAA = None
  localAA = None

  if platform.system() == "Windows":
    raise RuntimeError('CAD score not available on Windows')

  if mode == "classic":
    cad_calc_path = None
    cad_read_g_path = None
    cad_read_l_path = None
    if cad_bin_path:
      cad_calc_path = settings.Locate('CADscore_calc.bash', 
                                      search_paths=[cad_bin_path],
                                      search_system_paths=False)  
      cad_read_g_path = settings.Locate('CADscore_read_global_scores.bash', 
                                        search_paths=[cad_bin_path],
                                        search_system_paths=False)  
      cad_read_l_path=settings.Locate('CADscore_read_local_scores.bash', 
                                      search_paths=[cad_bin_path],
                                      search_system_paths=False)
      # also try to locate the actual executable that is called from the 
      # bash scripts 
      executable_path = settings.Locate('voroprot2', 
                                        search_paths=[cad_bin_path],
                                        search_system_paths=False) 
    else:
      cad_calc_path = settings.Locate('CADscore_calc.bash')  
      cad_read_g_path = settings.Locate('CADscore_read_global_scores.bash')  
      cad_read_l_path = settings.Locate('CADscore_read_local_scores.bash')
      # also try to locate the actual executable that is called from the 
      # bash scripts
      executable_path = settings.Locate('voroprot2')  
    command1="\"%s\" -m \"%s\" -t \"%s\" -D \"%s\"" %(cad_calc_path, 
                                                      model_filename, 
                                                      reference_filename, 
                                                      os.path.join(tmp_dir,
                                                                   "cadtemp"))
    command2="\"%s\" -D \"%s\"" %(cad_read_g_path, os.path.join(tmp_dir,
                                                                "cadtemp"))
    command3="\"%s\" -m \"%s\" -t \"%s\" -D \"%s\" -c AA" %(cad_read_l_path, 
                                                            model_filename, 
                                                            reference_filename,
                                                            os.path.join(tmp_dir,
                                                            "cadtemp"))

    ps1=subprocess.Popen(command1, shell=True, stdout=subprocess.PIPE)
    ps1.communicate()
    ps2=subprocess.Popen(command2, shell=True, stdout=subprocess.PIPE)
    stdout,_ = ps2.communicate()
    lines=stdout.decode().splitlines()
    try:
      globalAA=_ParseCADGlobal(lines)
    except:
      raise RuntimeError("CAD calculation failed")
    ps3=subprocess.Popen(command3, shell=True, stdout=subprocess.PIPE)
    stdout,_ = ps3.communicate()
    lines=stdout.decode().splitlines()
    try:
      localAA=_ParseCADLocal(lines)
    except:
      raise RuntimeError("CAD calculation failed")

  elif mode == "voronota":
    local_score_filename = os.path.join(tmp_dir, "local_scores.txt")
    voronota_cadscore_path = None
    if cad_bin_path:
      voronota_cadscore_path = settings.Locate("voronota-cadscore", 
                                               search_paths=[cad_bin_path],
                                               search_system_paths=False)
      # also try to locate the actual executable that is called from the 
      # bash script
      executable_path = settings.Locate("voronota", 
                                        search_paths=[cad_bin_path],
                                        search_system_paths=False)      
    else:
      voronota_cadscore_path = settings.Locate("voronota-cadscore")
      # also try to locate the actual executable that is called from the 
      # bash script
      executable_path = settings.Locate("voronota")  
    cmd = [voronota_cadscore_path, '-m', model_filename, '-t', 
           reference_filename, '--contacts-query-by-code AA', 
           '--output-residue-scores', local_score_filename]
    if old_regime:
      cmd.append("--old-regime")
    cmd = ' '.join(cmd)
    ps = subprocess.Popen(cmd, shell=True, stdout=subprocess.PIPE)
    stdout, _ = ps.communicate()
    try:
      globalAA = _ParseVoronotaGlobal(stdout.decode().splitlines())
    except:
      raise RuntimeError("CAD calculation failed")
    try:
      with open(local_score_filename) as f: 
        localAA = _ParseVoronotaLocal(f.readlines())
    except:
      raise RuntimeError("CAD calculation failed")

  else:
    raise RuntimeError("Invalid CAD mode! Allowed are: "
                       "[\"classic\", \"voronota\"]")


  return CADResult(globalAA,localAA)

def _HasInsertionCodes(model, reference):
  for r in model.residues:
    if r.GetNumber().GetInsCode() != "\0":
      return True
  for r in reference.residues:
    if r.GetNumber().GetInsCode() != "\0":
      return True
  return False

def _MapLabels(model, cad_results, label):
  for k,v in cad_results.localAA.items():
    r = model.FindResidue(k[0], k[1])
    if not r.IsValid():
      raise RuntimeError("Failed to map cadscore on residues: " +
                         "CAD score estimated for residue in chain \"" +
                         k[0] + "\" with ResNum " + str(k[1]) + ". Residue " +
                         "could not be found in model.")
    r.SetFloatProp(label, v)

def CADScore(model, reference, mode = "voronota", label = "localcad",
             old_regime = False, cad_bin_path = None):
  """
  Calculates global and local atom-atom (AA) CAD Scores. 

  You can either access the original implementation available from
  https://bitbucket.org/kliment/cadscore/downloads/
  or the new implementation which is part of the Voronota package 
  available from https://bitbucket.org/kliment/voronota/downloads/.

  The scores of the two implementations differ but strongly correlate
  as the contacts between atoms are estimated differently. When using
  the "voronota" *mode* you can minimize those discrepancies by
  setting the *old_regime* flag to True.

  Furthermore, the "voronota" *mode* generates per-residue scores that 
  are inverted when compared to the classical implementation 
  (0.0: bad, 1.0 good). 

  :param model: The model structure. 
  :type model: :class:`~ost.mol.EntityView` or :class:`~ost.mol.EntityHandle`
  :param reference: The reference structure
  :type reference: :class:`~ost.mol.EntityView` or 
                   :class:`~ost.mol.EntityHandle`
  :param mode:  What CAD score implementation to use, must be one in 
                ["classic", "voronota"]
  :param label: Local CAD scores will be mapped on residues of model as 
                float property with this name
  :type label: :class:`str`
  :param old_regime: Only has an effect if *mode* is "voronota". If set to true,
                     the discrepancies between the two modes is minimized but
                     the behaviour of inverted scores persists.
  :type old_regime: :class:`bool`
  :param cad_bin_path: Path to search for the required executables 
                       (["CADscore_calc.bash", 
                       "CADscore_read_global_scores.bash",
                       "CADscore_read_local_scores.bash"] for "classic" *mode* 
                       or ["voronota-cadscore"] for "voronota" *mode*). If not
                       set, the env path is searched.
  :type cad_bin_path: :class:`str`
  :returns: The result of the CAD score calculation
  :rtype: :class:`CADResult`
  
  :raises: :class:`~ost.settings.FileNotFound` if any of the CAD score 
           executables could not be located.
  :raises: :class:`RuntimeError` if the calculation failed
  """
  if mode == "classic" and _HasInsertionCodes(model, reference):
    raise RuntimeError("The classic CAD score implementation does not support "
                       "insertion codes in residues")
  tmp_dir_name=_SetupFiles(model, reference)
  result=_RunCAD(tmp_dir_name, mode, cad_bin_path, old_regime)
  _CleanupFiles(tmp_dir_name)
  _MapLabels(model, result, label)
  return result



from ost.seq.alg import SubstWeightMatrix

def _InitMatrix(preset_enum):
  mat = SubstWeightMatrix()
  mat.AssignPreset(preset_enum)
  return mat

BLOSUM45 = _InitMatrix(SubstWeightMatrix.Preset.BLOSUM45)
BLOSUM62 = _InitMatrix(SubstWeightMatrix.Preset.BLOSUM62)
BLOSUM80 = _InitMatrix(SubstWeightMatrix.Preset.BLOSUM80)
BLOSUM100 = _InitMatrix(SubstWeightMatrix.Preset.BLOSUM100)

__all__=['BLOSUM45','BLOSUM62','BLOSUM80','BLOSUM100']



#------------------------------------------------------------------------------
# This file is part of the OpenStructure project <www.openstructure.org>
#
# Copyright (C) 2008-2020 by the OpenStructure authors
#
# This library is free software; you can redistribute it and/or modify it under
# the terms of the GNU Lesser General Public License as published by the Free
# Software Foundation; either version 3.0 of the License, or (at your option)
# any later version.
# This library is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
# FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public License for more
# details.
#
# You should have received a copy of the GNU Lesser General Public License
# along with this library; if not, write to the Free Software Foundation, Inc.,
# 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
#------------------------------------------------------------------------------

import urllib.request, urllib.error, urllib.parse
import tempfile

from ost.io import LoadPDB, LoadMMCIF

class RemoteRepository:
  """
  A remote repository represents a structural database accessible through the 
  internet, e.g. the PDB or SWISS-MODEL template library.

  :param name:        Name of the repository
  :param url_pattern: URL pattern for repository. Required format is described
                      in :func:`URLForID`
  :param type:        Data format to expect at resolved URL must be in 
                      ('pdb', 'cif')
  :param id_transform: Transformation to apply to ID before resolving URL
                       in :func:`URLForID`. Must be in ('lower', 'upper')

  :type name:         :class:`str`
  :type url_pattern:  :class:`str`
  :type type:         :class:`str`
  :type id_transform: :class:`str`
  """
  def __init__(self, name, url_pattern, type, id_transform='upper'):
    self.name = name
    self.url_pattern = url_pattern
    self.type = type
    if type not in ('cif', 'pdb'):
      raise ValueError('only cif and pdb types are supported')
    self.id_transform = id_transform

  def URLForID(self, id):
    """
    Resolves URL given *url_pattern* and *id_transform* provided at object 
    initialization.
    The *url_pattern* must contain substring '$ID'. Given *id*, the URL to 
    the structure gets constructed by applying *id_transform* and inserting it
    at the location of '$ID'. e.g. 'https://files.rcsb.org/view/$ID.pdb' given 
    1ake as *id* and 'upper' as *id_transform* resolves to:
    'https://files.rcsb.org/view/1AKE.pdb'
    """
    if self.id_transform == 'upper':
      id = id.upper()
    if self.id_transform == 'lower':
      id = id.lower()
    return self.url_pattern.replace('$ID', id)

  def Get(self, id):
    """
    Resolves URL with :func:`URLForID`, dumps the content in a temporary file 
    and returns its path.
    
    :param id:          ID to resolve
    :type id:           :class:`str`
    """
    remote_url = self.URLForID(id)
    tmp_file_suffix = '.%s' % self.type
    if remote_url.endswith('.gz'):
      tmp_file_suffix+='.gz'

    try:
      connection = urllib.request.urlopen(remote_url)
      if hasattr(connection, 'code'):
        status = connection.code
      else:
        status = connection.getcode()
    except urllib.error.HTTPError as e:
      status = e.code
    if status != 200:
      raise IOError('Could not load %s from %s (status code %d, url %s)' \
                    % (id, self.name, status, remote_url))
    tmp_file = tempfile.NamedTemporaryFile(suffix=tmp_file_suffix)
    tmp_file.write(connection.read())
    tmp_file.flush()
    return tmp_file

  def Load(self, id):
    """
    Resolves URL with :func:`URLForID` and directly loads/returns the according 
    :class:`ost.mol.EntityHandle`. Loading invokes the 
    :func:`ost.io.LoadPDB`/:func:`ost.io.LoadMMCIF` with default parameterization. If you need
    custom settings, you might want to consider to call :func:`Get` and do the
    loading manually.
    
    :param id:          ID to resolve
    :type id:           :class:`str`
    """
    tmp_file = self.Get(id)
    if self.type == 'pdb':
      return LoadPDB(tmp_file.name)
    if self.type == 'cif':
      return LoadMMCIF(tmp_file.name)

REMOTE_REPOSITORIES = {
    'pdb' : RemoteRepository('rcsb.org (PDB)', 'https://files.rcsb.org/download/$ID.pdb.gz',
                   type='pdb', id_transform='upper'),
    'smtl' : RemoteRepository('SMTL', 'https://swissmodel.expasy.org/templates/$ID.pdb',
                   type='pdb', id_transform='lower'),
    'cif' : RemoteRepository('rcsb.org (mmCIF)', 'https://files.rcsb.org/download/$ID.cif.gz',
                   type='cif', id_transform='lower'),
    'pdb_redo' : RemoteRepository('pdbredo', 'https://pdb-redo.eu/db/$ID/$ID_besttls.pdb.gz',
                   type='pdb', id_transform='lower'),
}

def RemoteGet(id, from_repo='pdb'):
  """
  Invokes :func:`RemoteRepository.Get` on predefined repositories 
  ('pdb', 'smtl', 'cif', 'pdb_redo')

  :param from_repo:    One of the predefined repositories
  :type from_repo:     :class:`str`
  """
  remote_repo = REMOTE_REPOSITORIES.get(from_repo, None) 
  if not remote_repo:
    raise ValueError('%s is not a valid repository' % from_repo)
  return remote_repo.Get(id)

def RemoteLoad(id, from_repo='pdb'):
  """
  Invokes :func:`RemoteRepository.Load` on predefined repositories 
  ('pdb', 'smtl', 'cif', 'pdb_redo')

  :param from_repo:    One of the predefined repositories
  :type from_repo:     :class:`str`
  """
  remote_repo = REMOTE_REPOSITORIES.get(from_repo, None) 
  if not remote_repo:
    raise ValueError('%s is not a valid repository' % from_repo)
  return remote_repo.Load(id)



from ost import gui
from ost import gfx
import os
import ost
from PyQt5 import QtCore
from .preset import Preset

class PresetEditorListModel(QtCore.QAbstractListModel):
  def __init__(self, preset, parent=None, *args): 
    QtCore.QAbstractListModel.__init__(self, parent, *args)
    
    self.preset_ = preset
    
  def AddItem(self, op, row):
    self.insertRow(row, QtCore.QModelIndex(),op)
    model_index = self.createIndex(row,0)
    end_index = self.createIndex(self.rowCount(),0)
    self.emit(QtCore.SIGNAL("dataChanged"),model_index, end_index)
    return True
  
  def RemoveItem(self, row):
    self.removeRow(row, QtCore.QModelIndex())
    model_index = self.createIndex(row,0)
    self.emit(QtCore.SIGNAL("dataChanged"),model_index, model_index)
    return True
    
  def GetOp(self, model_index):
    if model_index.isValid():
      return self.preset_.GetOp(model_index.row())
  
  def SetItem(self, model_index, op):
    self.preset_.SetOp(model_index.row(), op)
  
  def GetLastRow(self):
    return self.rowCount()
    
  #Overwritten Methods  
  def rowCount(self, parent=QtCore.QModelIndex()): 
    return self.preset_.GetOpCount() 
 
  def data(self, index, role):
    if index.isValid() and index.row()< self.rowCount():
      data = self.preset_.GetOp(index.row())
      if role == QtCore.Qt.DisplayRole:
        name=data.GetName()
        selection=str(data.GetSelection())
        if(len(selection)>0):
          return QtCore.QVariant(str("%s (%s)"%(name,selection.replace(os.linesep, " "))))
        else:
          return QtCore.QVariant(str("%s (all)"%name))
    return QtCore.QVariant()

  def setData(self, index, value, role):
    if index.isValid():
      row = index.row()
      data = self.preset_.GetOp(row)
      if role == QtCore.Qt.EditRole:
        return True
      elif role == QtCore.Qt.DisplayRole:
        data
      elif role == QtCore.Qt.DecorationRole:
        data
    return False

  def flags(self, index):
    if index.isValid():
      flags = QtCore.QAbstractItemModel.flags(self,index)
      return flags | QtCore.Qt.ItemIsEditable
    return QtCore.Qt.ItemIsEnabled
  
  def insertRow(self, position, index, op):
    self.beginInsertRows(index, position, position)
    self.preset_.InsertOp(position, op)
    self.endInsertRows()
    return True
  
  def removeRow(self, position, index):
    self.beginRemoveRows(index, position, position)
    self.preset_.RemoveOpAt(position)
    self.endRemoveRows()
    return True
  



import subprocess, os, tempfile
from ost import io, seq, settings

"""
Wrapper for kClust a protein sequence clustering algorithm

unpublished, but mentioned in:

Michael Remmert, Andreas Biegert, Andreas Hauser & Johannes Soeding
HHblits: lighting-fast iterative protein sequence searching by
HMM-HMM alignment
Nature Methods 9, 173-175 (2012)

source code can be downloaded from:

ftp://toolkit.genzentrum.lmu.de/pub/
"""

class cluster:

  """
  Holds the information of one cluster
  
  .. attribute:: sequences
    
    SequenceList containing all sequences of the cluster
  
  .. attribute:: representative_id
  
    a string, that contains the name of the representative sequence of the cluster
    as estimated by kClust. 
    
  .. attribute:: alignment

    alignment handle containing a multiple sequence alignment of all sequences of
    the cluster. Gets only calculated when binding is called with appropriate flag.
  """

  def __init__(self, sequences, representative_id):

    self.sequences=sequences
    self.representative_id=representative_id
    self.alignment=None

def _SetupFiles(sequences):

  # create temporary directory
  tmp_dir_name=tempfile.mkdtemp()
  io.SaveSequenceList(sequences, os.path.join(tmp_dir_name,'fastadb.fasta'))
  return tmp_dir_name

def _CleanupFiles(tmp_dir_name):

  import shutil
  shutil.rmtree(tmp_dir_name)

def _ParseOutput(tmp_dir_name):

  with open(os.path.join(tmp_dir_name,'headers.dmp'),'r') as f:
    header_data=f.readlines()
  with open(os.path.join(tmp_dir_name,'clusters.dmp'),'r') as f:
    cluster_data=f.readlines()
  sequences=io.LoadSequenceList(os.path.join(tmp_dir_name,'fastadb.fasta'))

  clusters=dict()
  header_mapper=dict()
  for line in header_data:
    header_mapper[int(line.split()[0])]=line.split()[1].strip().strip('>')

  #find numeric ids of the representatives of the clusters
  unique_representatives=list()
  for line in cluster_data[1:]:
    actual_cluster=int(line.split()[1])
    try:
      unique_representatives.index(actual_cluster)
    except:
      unique_representatives.append(actual_cluster)

  #assign every header to its corresponding cluster, where the
  #cluster id is given by the id of the representative of the cluster
  for idx in unique_representatives:
    clusters[idx]=seq.CreateSequenceList()
  for line in cluster_data[1:]:
    clusters[int(line.split()[1])].AddSequence(sequences.FindSequence(header_mapper[int(line.split()[0])]))

  #translate into final output

  res=list()
  for k, v in clusters.items():
    res.append(cluster(v, header_mapper[k]))

  return res
  
def _RunkClust(tmp_dir_name, clustering_thresh, create_alignments):

  bitscore=clustering_thresh*0.060269-0.68498

  executable=settings.Locate('kClust') 

  cmd=[]
  cmd.append(executable)
  cmd.append('-i')
  cmd.append(os.path.join(tmp_dir_name,'fastadb.fasta'))
  cmd.append('-d')
  cmd.append(tmp_dir_name)
  cmd.append('-s')
  cmd.append(str(bitscore))

  cmd=' '.join(cmd)
  ps=subprocess.Popen(cmd, shell=True, stdout=subprocess.PIPE, stderr=subprocess.PIPE)
  stdout, stderr = ps.communicate()

  result=_ParseOutput(tmp_dir_name)

  if(create_alignments):
    from ost.bindings import clustalw
    for c in result:
      if len(c.sequences)>1:
        c.alignment=clustalw.ClustalW(c.sequences)
      else:
        aln=seq.CreateAlignment()
        aln.AddSequence(c.sequences[0])
        c.alignment=aln

  return result

def kClust(sequences, clustering_thresh=30, create_alignments=False):

  """
  Uses kClust to generate clusters of amino acid sequences.
  
  :param sequences: All sequences you want to cluster. 
  :type sequences: :class:`ost.seq.SequenceList`

  :param clustering_thres: Sequence identity threshold to build clusters. Note,
                           that clustering_thresh is more a rule of thumb, since
                           compositional bias in the sequence can also play a role.
                           The value gets transformed in a bitscore, that is used
                           as an input parameter of kClust 

  :param create_alignments: Flag, wether the alignments of the clusters get calculated.
                            Requires clustalw in the path.

  :returns: A list of cluster instances
  
  :raises: :class:`~ost.settings.FileNotFound` if kClust could not be located.
  """

  tmp_dir_name=_SetupFiles(sequences)
  result=_RunkClust(tmp_dir_name, clustering_thresh, create_alignments)
  _CleanupFiles(tmp_dir_name)
  return result



import ost
from ost.mol.alg import qsscoring

# load two biounits to compare
ent_full = ost.io.LoadPDB('3ia3', remote=True)
ent_1 = ent_full.Select('cname=A,D')
ent_2 = ent_full.Select('cname=B,C')
# get score
ost.PushVerbosityLevel(3)
try:
    qs_scorer = qsscoring.QSscorer(ent_1, ent_2)
    ost.LogScript('QSscore:', str(qs_scorer.global_score))
    ost.LogScript('Chain mapping used:', str(qs_scorer.chain_mapping))
except qsscoring.QSscoreError as ex:
    # default handling: report failure and set score to 0
    ost.LogError('QSscore failed:', str(ex))
    qs_score = 0

print("OST is working!")



#------------------------------------------------------------------------------
# This file is part of the OpenStructure project <www.openstructure.org>
#
# Copyright (C) 2008-2020 by the OpenStructure authors
#
# This library is free software; you can redistribute it and/or modify it under
# the terms of the GNU Lesser General Public License as published by the Free
# Software Foundation; either version 3.0 of the License, or (at your option)
# any later version.
# This library is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
# FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public License for more
# details.
#
# You should have received a copy of the GNU Lesser General Public License
# along with this library; if not, write to the Free Software Foundation, Inc.,
# 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
#------------------------------------------------------------------------------

from ost import gui

from PyQt5 import QtCore, QtGui, QtWidgets

#Get Panels (Class which manages widgets)
panels=gui.GostyApp.Instance().perspective.panels

#Create Widget
my_awesome_widget = QtWidgets.QWidget()
layout = QtWidgets.QVBoxLayout()
layout.addWidget(QtWidgets.QPushButton('I dont do anything'))
layout.addWidget(QtWidgets.QPushButton('I do even less'))
my_awesome_widget.setLayout(layout)

#Wrap widget to Qt Widget
wid=gui.Widget(my_awesome_widget)

#Add Widget to widget pool
panels.AddWidgetToPool("Widget that doesnt do a thing", wid)

#Add Widget to right panel
panels.AddWidget(gui.PanelPosition.RIGHT_PANEL,wid,False)




#------------------------------------------------------------------------------
# This file is part of the OpenStructure project <www.openstructure.org>
#
# Copyright (C) 2008-2020 by the OpenStructure authors
#
# This library is free software; you can redistribute it and/or modify it under
# the terms of the GNU Lesser General Public License as published by the Free
# Software Foundation; either version 3.0 of the License, or (at your option)
# any later version.
# This library is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
# FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public License for more
# details.
#
# You should have received a copy of the GNU Lesser General Public License
# along with this library; if not, write to the Free Software Foundation, Inc.,
# 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
#------------------------------------------------------------------------------
# -*- coding: utf-8 -*-

from ost import gui
from ost import gfx
from ost import mol
try: 
  from ost import img
  _img_present=True
except ImportError:
  _img_present=False
  pass
from PyQt5 import QtCore, QtWidgets, QtGui
from .color_select_widget import ColorSelectWidget

#Uniform Color Widget
class UniformColorWidget(QtWidgets.QWidget):
  def __init__(self, parent=None):
    QtWidgets.QWidget.__init__(self, parent)
    self.parent_ = parent
    
    self.text_ = "Uniform Color"
    
    #Create Ui elements
    uniform_label = QtWidgets.QLabel(self.text_)
    font = uniform_label.font()
    font.setBold(True)
    
    self.color_select_widget_ = ColorSelectWidget(1,1,QtGui.QColor("White"))
    
    top_layout = QtWidgets.QVBoxLayout()
    
    grid = QtWidgets.QGridLayout()
    grid.addWidget(self.color_select_widget_, 2, 1, 1, 1)
    grid.setRowStretch(1, 1)
    grid.setRowStretch(3, 1)
    grid.setColumnStretch(0,1)
    grid.setColumnStretch(2,1)
    
    top_layout.addWidget(uniform_label)
    top_layout.addLayout(grid)
    self.setLayout(top_layout)
    
    self.color_select_widget_.colorChanged.connect(self.ChangeColors)
    
    self.setMinimumSize(250,150)

  def Update(self):
    scene_selection = gui.SceneSelection.Instance()
    for i in range(0,scene_selection.GetActiveNodeCount()):
      node = scene_selection.GetActiveNode(i)
      if _img_present and isinstance(node, gfx.MapIso):
        if self.color_select_widget_.GetGfxColor() != node.GetColor():
          self.color_select_widget_.SetGfxColor(node.GetColor())
      else:
        self.ChangeColors()
        
  def ChangeColors(self):
    scene_selection = gui.SceneSelection.Instance()
    for i in range(0,scene_selection.GetActiveNodeCount()):
      node = scene_selection.GetActiveNode(i)
      self.ChangeColor(node)
    
    if(scene_selection.GetActiveViewCount() > 0):
      entity = scene_selection.GetViewEntity()
      view = scene_selection.GetViewUnion()
      self.ChangeViewColor(entity,view)
  
  def ChangeColor(self, node):
    gfx_color = self.color_select_widget_.GetGfxColor()
    if isinstance(node, gfx.Entity) or isinstance(node, gfx.Surface):
        node.CleanColorOps()
        if self.parent_.GetCarbonsOnly():
          node.SetColor(gfx_color,"ele=C")
        else:
          node.SetColor(gfx_color,"")
    elif _img_present and isinstance(node, gfx.MapIso):
        node.SetColor(gfx_color)
  
  def ChangeViewColor(self, entity, view):
    if isinstance(entity, gfx.Entity) and isinstance(view, mol.EntityView):
      gfx_color = self.color_select_widget_.GetGfxColor()
      if self.parent_.GetCarbonsOnly():
        ufco=gfx.UniformColorOp(mol.QueryViewWrapper(mol.Query("ele=C"), view),gfx_color)
      else:
        ufco=gfx.UniformColorOp(mol.QueryViewWrapper(view),gfx_color)
      entity.Apply(ufco)
    
  def resizeEvent(self, event):
    self.color_select_widget_.SetSize(self.width()/2,self.height()/2)
    
  def GetText(self):
    return self.text_



s=io.LoadSurface("../demos/data/sh2","msms")
scene.Add(gfx.Surface("s",s))

# alternative shading mode
scene.SetShadingMode("hf")

# add outlines to surface
scene["s"].SetOutline(True)
scene["s"].SetOutlineMode(3)

# turn on realtime ambient occlusion
scene.SetAmbientOcclusion(True)

# strength of the AO effect, default is 1.0
scene.SetAmbientOcclusionWeight(1.0)

# AO algorithms 0-3, default is 1
scene.SetAmbientOcclusionMode(1)




from ._ost_seq_alg import *
from ost.seq.alg.mat import *

def ValidateSEQRESAlignment(aln, chain=None):
  """
  Checks if sequence in alignment has same connectivity as residues in chain.
  This looks for connected stretches in both the sequence and the chain and
  returns False if they don't match. This uses the connectivity of the protein
  backbone.

  :param aln: Alignment of two sequences with second one expected to map to
              residues in *chain*.
  :type aln: :class:`~ost.seq.AlignmentHandle`
  :param chain: Source of the sequence.
  :type chain: :class:`~ost.mol.ChainHandle`

  :returns: True if all residues (beside gapped ones) are connected, False
            otherwise.
  """
  from ost import LogWarning
  from ost import seq
  from ost import mol
  if aln.GetCount() != 2:
    raise ValueError('Alignment contains more than 2 sequences!')
  sequence = aln.GetSequence(1)
  if len(sequence) == 0:
    return True
  if chain == None:
    if sequence.HasAttachedView() == False:
      raise ValueError("Alignment is missing an attached chain view.")
    chain = sequence.GetAttachedView()
  residues = chain.residues
  # eat up all beginning gaps
  j = 1
  for s in sequence:
    if s != '-':
      break
    j += 1;
  l = sequence[j-1]
  i = 0
  # run over sequence & alignment
  for s in sequence[j:]:
    if s != '-':
      i += 1
      r1 = residues[i-1]
      r2 = residues[i]
      if r1.one_letter_code=='?' or r2.one_letter_code=='?':
        continue
      if l != '-':
        if not mol.InSequence(r1.handle, r2.handle):
          LogWarning('%s and %s are not connected by peptide bond' % (str(r1), str(r2)))
          return False
      else:
        if mol.InSequence(r1.handle, r2.handle):
          LogWarning('%s and %s are connected by peptide bond' % (str(r1), str(r2)))
          return False
    l = s
  return True

def AlignToSEQRES(chain, seqres, try_resnum_first=False, validate=True):
  """
  Aligns the residues of chain to the SEQRES sequence, inserting gaps where 
  needed. The function uses the connectivity of the protein backbone to find 
  consecutive peptide fragments. These fragments are then aligned to the SEQRES 
  sequence.
  
  All the non-ligand, peptide-linking residues of the chain must be listed in 
  SEQRES. If there are any additional residues in the chain, the function
  raises a ValueError.

  :param chain: Source of the sequence
  :type chain: :class:`~ost.mol.ChainHandle`
  :param seqres: SEQRES sequence
  :type seqres: :class:`str`
  :param try_resnum_first: If set to True, this first builds an alignment using
                           residue numbers and checks if the one-letter-codes
                           match. If they all match, this alignment is used
                           (and possibly validated). Otherwise, it displays a
                           warning and falls back to the connectivity-based
                           alignment.
  :type try_resnum_first: :class:`bool`
  :param validate: If set to True, the alignment is additionally checked by
                   :func:`~ost.seq.alg.ValidateSEQRESAlignment` and raises
                   a ValueError if the validation failed.
  :type validate: :class:`bool`

  :returns: The alignment of the residues in the chain and the SEQRES entries.
  :rtype: :class:`~ost.seq.AlignmentHandle`
  """

  def IsEqual(olc1, olc2):
    return olc1 in ('X', '?') or olc2 in ('X', '?') or olc1 == olc2

  from ost import seq
  from ost import mol
  from ost import LogWarning
  view=chain
  residues=view.residues
  if len(residues)==0:
    return seq.CreateAlignment()
  if try_resnum_first:
    aln_seq = seq.CreateSequence('atoms', '-'*len(seqres))
    aligned_resnums = set()
    for r1 in residues:
      if r1.number.num in aligned_resnums:
        LogWarning('Residue numbers must be unique. Already observed %i, ' \
                   'cannot align %s anymore.'%(r1.number.num, r1.qualified_name))
        try_resnum_first = False
        break
      if r1.number.num <= len(seqres) and r1.number.num > 0:
        if IsEqual(seqres[r1.number.num - 1], r1.one_letter_code):
          aln_seq[r1.number.num - 1] = r1.one_letter_code
          aligned_resnums.add(r1.number.num)
        else:
          LogWarning('Sequence mismatch: chain has "' + r1.one_letter_code +
                     '", while SEQRES is "' + seqres[r1.number.num - 1] +
                     '" at the corresponding position.')
          try_resnum_first = False
          break
      else:
        warning = 'Residue with number %i is outside of the range covered by '\
                  'SEQRES [1, %i]'%(r1.number.num, len(seqres)) 
        LogWarning(warning)
        try_resnum_first = False
        break
  if not try_resnum_first:
    fragments=[residues[0].one_letter_code]
    for r1, r2 in zip(residues[:-1], residues[1:]):
      if not mol.InSequence(r1.handle, r2.handle):
        fragments.append('')
      fragments[-1]+=r2.one_letter_code
    ss=str(seqres)
    pos=0
    aln_seq=''
    for frag in fragments:
      new_pos=ss.find(frag, pos)
      if new_pos==-1:
        raise ValueError('"%s" is not a substring of "%s"' % (frag, ss))
      aln_seq+='-'*(new_pos-pos)+frag
      pos=new_pos+len(frag)
    aln_seq = seq.CreateSequence('atoms',
                                 aln_seq+('-'*(len(seqres)-len(aln_seq))))
  alignment = seq.CreateAlignment(seq.CreateSequence('SEQRES', str(seqres)), 
                                  aln_seq)
  if validate and not ValidateSEQRESAlignment(alignment, view):
    raise ValueError("SEQRES cannot be aligned with its corresponding chain.")
  return alignment


def AlignmentFromChainView(chain, handle_seq_name='handle', 
                           view_seq_name='view'):
  """
  Creates and returns the sequence alignment of the given chain view to the 
  chain handle. The alignment contains two sequences, the first containing all 
  non-ligand peptide-linking residues, the second containing all non-ligand 
  peptide-linking residues that are part of the view. 
  
  :param chain: A valid chain
  :type chain: :class:`~ost.mol.ChainView`
  
  :param handle_seq_name: Name of the handle sequence in the output alignment
  :param view_seq_name: Name of the view sequence in the output alignment
  :returns: The alignment
  :rtype: :class:`~ost.seq.AlignmentHandle`
  
  """
  from ost import seq
  v0=chain.handle.Select('ligand=false and peptide=true')
  v1=chain.Select('ligand=false and peptide=true')
  s0=seq.CreateSequence(handle_seq_name, '')
  s1=seq.CreateSequence(view_seq_name, '')
  s0.AttachView(v0)
  s1.AttachView(v1)
  res0=v0.residues
  res1=v1.residues
  idx0, idx1=(0, 0)
  while idx0<len(res0):
    s0.Append(res0[idx0].one_letter_code)
    if idx1<len(res1) and res1[idx1].handle==res0[idx0].handle:
      s1.Append(res1[idx1].one_letter_code)
      idx1+=1
    else:
      s1.Append('-')
    idx0+=1
  return seq.CreateAlignment(s0, s1)

def PredictContacts(ali):
  """
  Predicts contacts from a multiple sequence alignment using a combination
  of Mutual Information (*MI*) and the Contact Substitution Score (*CoEvoSc*).
  MI is calculated with the APC and small number corrections as well as with a 
  transformation into Z-scores. The *CoEvoSc* is calculated using the default 
  PairSubstWeightMatrix (see seq.alg.LoadDefaultPairSubstWeightMatrix).
  The final score for a pair of columns *(i,j)* of **ali** is obtained from:
  
  Sc(i,j)=MI(i,j)exp(CoEvoSc(i,j))      if *(i,j)* >=0
  
  Sc(i,j)=MI(i,j)exp(1-CoEvoSc(i,j))    if *(i,j)* <0

  :param ali: The multiple sequence alignment
  :type ali: :class:`~ost.seq.AlignmentHandle`
  """
  import math
  from ost import seq
  if not type(ali)==type(seq.AlignmentHandle()):
    print("Parameter should be an AlignmentHandle")
    return
  mi=CalculateMutualInformation(ali)
  CoEvoSc=CalculateContactSubstitutionScore(ali)
  ncol=ali.GetLength()
  for i in range(ncol):
    for j in range(ncol):
      if mi.matrix[i][j]>=0:
        mi.matrix[i][j]=mi.matrix[i][j]*(math.exp(CoEvoSc.matrix[i][j]))
      else:
        mi.matrix[i][j]=mi.matrix[i][j]*(math.exp(1.-CoEvoSc.matrix[i][j]))
  mi.RefreshSortedIndices()
  return mi

def CalculateContactProbability(cpred_res,method):
  """
  Calculate the probability of a predicted contact to be correct.
  This simply transforms the score associated with a prediction into a probability.

  :param cpred_res: A contact prediction result
  :param method: The method which was used for contact prediction. Should be one
   of "MI", "MIp", "MIpz", "cevoMI", "cevo"

  :type cpred_res: :class:`ost.seq.alg.ContactPredictionScoreResult`
  :type method: :class:`str`
  """
  import math
  def _growth_function(x,K,x0,B,nu,Q):
    p=K/(1+Q*math.exp(-B*(x-x0)))**(1/nu)
    p=min(1,max(0,p))
    return p
  
  def _decay_function(x,A,k):
    p=A*math.exp(-k*x)
    p=min(1,max(0,p))
    return p

  prob_params={}
  prob_params["MI"]=[0.05858455345122933, 0.8930350957023122]
  prob_params["MIp"]=[0.10019621004607637, 0.9065429261332942]
  prob_params["MIpz"]=[0.7368147563063437, 4.638820470770171, 0.6383539191372934, 1.1627300209863376, 19.63060874042289]
  prob_params["cevoMI"]=[0.779979757231944, 1.642357937131157, 0.3267847173036033, 0.3742848849873358, 5.1816922372446]
  prob_params["cevo"]=[0.7564532665623617, 0.5947472274271304, 3.8548775389879166, 0.46203017320927053, 1.6198602780123705]

  if not method in prob_params:
    raise ValueError("method should be one of MI, MIp, MIpz, cevoMI, cevo")
  params=prob_params[method]
  cpred_res.RefreshSortedIndices()
  nres=len(cpred_res.matrix)
  probabilities=[[0 for i in range(nres)] for j in range(nres)]

  if len(params)==2:func=_decay_function
  else:func=_growth_function

  nres=float(nres)
  if len(params)==2:
    for k,(i,j) in enumerate(cpred_res.sorted_indices):
      p=_decay_function(math.log10(0.01+k/nres),*params)
      probabilities[i][j]=p
      probabilities[j][i]=p
  else:
    for k,(i,j) in enumerate(cpred_res.sorted_indices):
      p=_growth_function(cpred_res.GetScore(i,j),*params)
      probabilities[i][j]=p
      probabilities[j][i]=p
  cpred_res.probabilities=probabilities
  return cpred_res







import unittest
import sys
from ost import *
from ost import settings
from ost.bindings import hbplus

class TestHBPlusBinding(unittest.TestCase):

  def setUp(self):
    self.protein = io.LoadEntity("testfiles/testprotein.pdb")

  def testHBondList(self):
    hbond_list = hbplus.HBondList(self.protein)
    self.assertEqual(len(hbond_list), 499)

  def testHBondScore(self):
    self.assertEqual(hbplus.HBondScore(self.protein, self.protein), 1.0)

if __name__ == "__main__":
  try:
    settings.Locate("hbplus")
  except:
    print("Could not find hbplus, could not test binding...")
    sys.exit(0)
  from ost import testutils
  testutils.RunTests()



#------------------------------------------------------------------------------
# This file is part of the OpenStructure project <www.openstructure.org>
#
# Copyright (C) 2008-2020 by the OpenStructure authors
#
# This library is free software; you can redistribute it and/or modify it under
# the terms of the GNU Lesser General Public License as published by the Free
# Software Foundation; either version 3.0 of the License, or (at your option)
# any later version.
# This library is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
# FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public License for more
# details.
#
# You should have received a copy of the GNU Lesser General Public License
# along with this library; if not, write to the Free Software Foundation, Inc.,
# 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
#------------------------------------------------------------------------------
from ._ost_mol import *
import ost.geom as _geom
from ost.mol import alg


def Transform(tf=None):
  from ost import LogWarning
  if Transform.mol_transform_warning_flag:
    LogWarning("mol.Transform is deprecated, please use geom.Transform instead")
    Transform.mol_transform_warning_flag=False
  if tf:
    return _geom.Transform(tf)
  else:
    return _geom.Transform()
Transform.mol_transform_warning_flag=True

def MergeCoordGroups(*coord_groups):
  """
  Merge several separate coord groups into one. The coord groups must have the 
  same number of atoms. In case no coord group is supplied, None will be 
  returned.
  """
  if len(coord_groups)==0:
    return None
  cg=CreateCoordGroup(coord_groups[0].atoms)
  for coord_group in coord_groups:
    cg.AddFrames(coord_group)
  return cg



from ost import gui
from ost import gfx
import os
import ost
from PyQt5 import QtCore, QtWidgets, QtGui
from .immutable_gradient_info_handler import ImmutableGradientInfoHandler
from .gradient_info_handler import GradientInfoHandler

class GradientListModel(QtCore.QAbstractListModel):
  
  IMMUTABLE_GRADIENTS_PATH = os.path.join(ost.GetSharedDataPath(),"scene", 
                                          "gradients.xml")
  MUTABLE_GRADIENTS_PATH = "user_gradients.xml"

  dataChanged = QtCore.pyqtSignal(int, int, name="dataChanged")
  
  def __init__(self, parent=None, *args): 
    QtCore.QAbstractListModel.__init__(self, parent, *args)
    
    self.data_ = list()
    
    #Info Handler
    self.immutable_infoh_ = ImmutableGradientInfoHandler(GradientListModel.IMMUTABLE_GRADIENTS_PATH)
    self.infoh_ = GradientInfoHandler(GradientListModel.MUTABLE_GRADIENTS_PATH)
    self.LoadGradientFromInfo()
    
  def AddItem(self, name, gradient, row, editable, save):
    if self.NameIsValid(name):
      icon = self.GetIcon(gradient)
      self.insertRow(row, QtCore.QModelIndex())
      self.data_[row] = [name, gradient, icon, editable]
      model_index = self.createIndex(row,0)
      index = self.index(row)
      end_index = self.createIndex(self.rowCount(),0)
      if save:
        self.AddGradientToInfo(gradient,name)
      self.dataChanged.emit(model_index, end_index)
      return True
    return False
  
  def IsEditable(self, row):
    return self.data_[row][3]
  
  def RemoveItem(self, row):
    if self.IsEditable(row):
      name = self.data_[row][0]
      self.removeRow(row, QtCore.QModelIndex())
      model_index = self.createIndex(row,0)
      self.infoh_.RemoveGradient(name)
      self.dataChanged.emit(model_index, model_index)
      return True
    return False

  def AddGradientToInfo(self,gradient,name):
    self.infoh_.StoreQGradient(gradient,str(name))
    
  def LoadGradientFromInfo(self):
    if self.immutable_infoh_:
      qgradients = self.immutable_infoh_.GetQGradients()
      for k, v in qgradients.items():
        self.AddItem(k, v, self.GetLastRow(), False, False)
    
    qgradients = self.infoh_.GetQGradients()
    for k, v in qgradients.items():
      self.AddItem(k, v, self.GetLastRow(), True, False)

  def GetGradient(self, model_index):
    if model_index.isValid():
      return self.data_[model_index.row()][1]
  
  def GetLastRow(self):
    return self.rowCount()
  
  #Helper
  def GetIcon(self, gradient):
    pixmap = QtGui.QPixmap(64, 64)
    pixmap.fill(QtCore.Qt.transparent)
    painter = QtGui.QPainter()
    if painter.begin(pixmap):
      gradient.setStart(QtCore.QPointF(0, 0))
      gradient.setFinalStop(QtCore.QPointF(64, 0))
      brush = QtGui.QBrush(gradient)
      painter.setBrush(brush)
      painter.drawRect(0, 0, 64, 64)
      painter.end()
    
    return QtGui.QIcon(pixmap)
  
  def NameIsValid(self, string):
    if len(string)==0:
      return False
    for values in self.data_:
      if string == values[0]:
        return False
    return True
  
  #Overwritten Methods  
  def rowCount(self, parent=QtCore.QModelIndex()): 
    return len(self.data_) 
 
  def data(self, index, role):
    if index.isValid() and index.row()< self.rowCount():
      data = self.data_[index.row()]
      if role == QtCore.Qt.DisplayRole:
        return QtCore.QVariant(data[0])
      elif role == QtCore.Qt.DecorationRole:
        return QtCore.QVariant(data[2])
    return QtCore.QVariant()

  def setData(self, index, value, role):
    if index.isValid():
      row = index.row()
      if not self.data_[row]:
        self.data_[row] = list()
      if role == QtCore.Qt.EditRole and self.NameIsValid(value.toString()):
        old_name = str(self.data_[row][0])
        new_name = value.toString()
        self.data_[row][0] = new_name
        self.infoh_.RenameGradient(old_name,str(new_name))
        self.dataChanged.emit(index, index)
        return True
      elif role == QtCore.Qt.DisplayRole:
        self.data_[row][0] = value.toString()
      elif role == QtCore.Qt.DecorationRole:
        self.data_[row][2] = value.toPyObject()
    return False

  def flags(self, index):
    if index.isValid():
      flags = QtCore.QAbstractItemModel.flags(self,index)
      if self.IsEditable(index.row()):
        return flags | QtCore.Qt.ItemIsEditable
      else:
        return flags
    return QtCore.Qt.ItemIsEnabled
  
  def insertRow(self, position, index):
    self.beginInsertRows(index, position, position)
    self.data_.insert(position,list())
    self.endInsertRows()
    return True
  
  def removeRow(self, position, index):
    self.beginRemoveRows(index, position, position)
    del self.data_[position]
    self.endRemoveRows()
    return True

  
  
  



from PyQt5.QtGui import *
from PyQt5.QtCore import *

from ost import table
__all__=('Table', )

class TableModel(QAbstractTableModel):
  def __init__(self, table, parent=None):
    QAbstractTableModel.__init__(self, parent)
    self.table=table
    
  def rowCount(self, index):
    return len(self.table.rows)

  def headerData(self, section, orientation, role):
    if role!=Qt.DisplayRole or orientation!=Qt.Horizontal:
      return QVariant()
    return self.table.col_names[section]
    
  def columnCount(self, index):
    return len(self.table.col_names)

  def sort(self, column, order):
    o='+'
    if order!=Qt.AscendingOrder:
      o='-'
    self.table.Sort(by=self.table.col_names[column], order=o)
    self.reset()
    
  def data(self, index, role):
    if not index.isValid() or role!=Qt.DisplayRole:
      return QVariant()
    row=self.table.rows[index.row()]
    return QVariant(row[index.column()])

class Table(QTableView):
  def __init__(self, table):
     QTableView.__init__(self)
     self._model=TableModel(table)
     self.setFrameShape(QFrame.NoFrame)    
     self.setAttribute(Qt.WA_MacSmallSize)
     self.setShowGrid(False)
     self.double_click = None
     self.horizontalHeader().setStretchLastSection(True)
     self.setContextMenuPolicy(Qt.CustomContextMenu)
     self.setSelectionBehavior(QAbstractItemView.SelectRows)
     self.setSizePolicy(QSizePolicy.MinimumExpanding, 
                        QSizePolicy.MinimumExpanding)
     self.setSortingEnabled(True)
     self.setModel(self._model)
     QObject.connect(self, SIGNAL('doubleClicked(QModelIndex)'), 
                     self.OnDoubleClick)
  def OnDoubleClick(self, model_index):
    print('DOUBLE')
    if not self.double_click:
      return
    row = table.TableRow(self._model.table.rows[model_index.row()],
                         self._model.table)
    self.double_click(row)



#------------------------------------------------------------------------------
# This file is part of the OpenStructure project <www.openstructure.org>
#
# Copyright (C) 2008-2020 by the OpenStructure authors
#
# This library is free software; you can redistribute it and/or modify it under
# the terms of the GNU Lesser General Public License as published by the Free
# Software Foundation; either version 3.0 of the License, or (at your option)
# any later version.
# This library is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
# FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public License for more
# details.
#
# You should have received a copy of the GNU Lesser General Public License
# along with this library; if not, write to the Free Software Foundation, Inc.,
# 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
#------------------------------------------------------------------------------
# -*- coding: utf-8 -*-

from ost import gui
from ost import gfx
from ost import info
from ost.gui.scene.preset import Preset

#Gradient Editor
class ImmutablePresetInfoHandler:
  PRESETS_GROUP_NAME = "Presets"
  PRESET_GROUP_NAME = "Preset"
  NAME_ATTRIBUTE_NAME = "Name"
  
  def __init__(self, file_name):
    self.file_name_ = file_name
    self.handle_ = info.LoadOrCreateInfo(self.file_name_)
    
    self.root_ = self.handle_.Root()
    
    self.presets_ = self.root_.RetrieveGroup(ImmutablePresetInfoHandler.PRESETS_GROUP_NAME)

    
  def GetPreset(self, name):
    group_list = self.presets_.GetGroups(ImmutablePresetInfoHandler.PRESET_GROUP_NAME)
    for group in group_list:
      if group.HasAttribute(ImmutablePresetInfoHandler.NAME_ATTRIBUTE_NAME):
        groupname = group.GetAttribute(ImmutablePresetInfoHandler.NAME_ATTRIBUTE_NAME)
        if name == groupname:
          preset = Preset.FromInfo(group)
          return preset
    return None
  
  def GetPresets(self):
    group_list = self.presets_.GetGroups(ImmutablePresetInfoHandler.PRESET_GROUP_NAME)
    presets = dict()
    
    for group in group_list:
      if group.HasAttribute(ImmutablePresetInfoHandler.NAME_ATTRIBUTE_NAME):
        preset = Preset.FromInfo(group)
        presets[group.GetAttribute(self.NAME_ATTRIBUTE_NAME)] = preset
    return presets



from ost import seq, mol

def _RenumberSeq(seq_handle, old_number_label=None):
  if not seq_handle.HasAttachedView():
    raise RuntimeError("Sequence Handle has no attached view")
  ev = seq_handle.attached_view.CreateEmptyView()
  new_numbers = mol.ResNumList()
  for pos in range(len(seq_handle)):
    if seq_handle[pos] != '-':
      r = seq_handle.GetResidue(pos)
      if r.IsValid():
          ev.AddResidue(r, mol.INCLUDE_ALL)
          new_numbers.append(pos+1)
          if old_number_label is not None:
            r.SetIntProp(old_number_label, r.number.GetNum())
      else:
        raise RuntimeError('Error: renumbering failed at position %s' % pos)
  return ev, new_numbers

def _RenumberAln(aln, seq_index, old_number_label=None):
  if not aln.sequences[seq_index].HasAttachedView():
    raise RuntimeError("Sequence Handle has no attached view")
  counter=0
  ev = aln.sequences[seq_index].attached_view.CreateEmptyView()
  new_numbers = mol.ResNumList()
  for col in aln:
    if col[0] != '-' and col[seq_index] != '-':
      if col[0] != col[seq_index]:
        raise RuntimeError("residue mismatch at position %d (%s vs %s) "\
                           "(renumbering failed)" % (counter, col[0],
                                                     col[seq_index]))
      rnum = aln.GetSequence(seq_index).GetResidueIndex(counter)
      r = aln.GetSequence(seq_index).GetResidue(counter)
      if not r.IsValid():
        raise RuntimeError("invalid residue at postion %s (renumbering failed)"\
                           % (counter))
      ev.AddResidue(r, mol.INCLUDE_ALL)
      new_numbers.append(counter+1)
      if old_number_label is not None:
        r.SetIntProp(old_number_label, r.number.GetNum())
    counter += 1
  return ev, new_numbers


def Renumber(seq_handle, sequence_number_with_attached_view=1, old_number_label=None):
  """
  Function to renumber an entity according to an alignment between the model
  sequence and the full-length target sequence. The aligned model sequence or
  the alignment itself with an attached view needs to be provided. Upon
  succcess, the renumbered entity is returned.
  If an alignment is given, the first sequence of the alignment is considered
  the full-length sequence and it must match the model sequence wherever it is
  aligned (i.e. excluding gaps).

  .. code-block:: python

    from ost.seq.alg import renumber
    from ost.bindings.clustalw import *
    ent = io.LoadPDB("path_to_model")
    s = io.LoadSequence("path_to_full_length_fasta_seqeunce")
    pdb_seq = seq.SequenceFromChain("model", ent.chains[0])
    aln = ClustalW(s, pdb_seq)
    aln.AttachView(1, ent.chains[0].Select(""))
    e = Renumber(aln.sequences[1])
    io.SavePDB(e, "renum.pdb")
   
  :param seq_handle: Sequence or alignment handle with attached view.
  :type seq_handle:  :class:`SequenceHandle` / :class:`AlignmentHandle`
  :param sequence_number_with_attached_view: Sequence number for the aln. handle
                                             (not used if seq. handle given)
  :type sequence_number_with_attached_view:  :class:`int`
  :raises: :exc:`RuntimeError` if unknown type of *seq_handle* or if attached
           view is missing or if the given alignment sequence is inconsistent.
  """
  if isinstance(seq_handle, seq.SequenceHandle) \
     or isinstance(seq_handle, seq.ConstSequenceHandle):
    ev, new_numbers = _RenumberSeq(seq_handle, old_number_label)
  elif isinstance(seq_handle, seq.AlignmentHandle):
    ev, new_numbers = _RenumberAln(seq_handle, sequence_number_with_attached_view, old_number_label)
  else:
    raise RuntimeError("Unknown input type " + str(type(seq_handle)))

  ev.AddAllInclusiveBonds()
  new_ent = mol.CreateEntityFromView(ev, False);
  new_ent.EditXCS().RenumberChain(new_ent.chains[0], new_numbers)
  return new_ent

# choose visible interface
__all__ = ('Renumber', )


#------------------------------------------------------------------------------
# This file is part of the OpenStructure project <www.openstructure.org>
#
# Copyright (C) 2008-2020 by the OpenStructure authors
#
# This library is free software; you can redistribute it and/or modify it under
# the terms of the GNU Lesser General Public License as published by the Free
# Software Foundation; either version 3.0 of the License, or (at your option)
# any later version.
# This library is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
# FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public License for more
# details.
#
# You should have received a copy of the GNU Lesser General Public License
# along with this library; if not, write to the Free Software Foundation, Inc.,
# 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
#------------------------------------------------------------------------------

import unittest
if __name__== '__main__':
  import sys
  sys.path.insert(0,"../../../stage/lib64/openstructure/")
  sys.path.insert(0,"../../../stage/lib/openstructure/")

import ost
import ost.geom as geom

# this terrible hack is required for older version of Python/Boost
# which do not convert std::out_of_range to IndexError correctly.
try:
  geom.Vec2()[3]
except Exception as e:
  pass
  IndexError = type(e)

class TestGeom(unittest.TestCase):
  def runTest(self):
    self.test_repr()
    self.test_data()

  def test_repr(self):
    v=geom.Vec2(1,2)
    v2=eval(repr(v))
    self.assertTrue(geom.Equal(v, v2))
                     
    v=geom.Vec3(1,2,3)
    v2=eval(repr(v))
    self.assertTrue(geom.Equal(v, v2))

    v=geom.Vec4(1,2,3,4)
    v2=eval(repr(v))
    self.assertTrue(geom.Equal(v, v2))

    m=geom.Mat2(1,2,3,4)
    m2=eval(repr(m))
    self.assertTrue(geom.Equal(m, m2))

    m=geom.Mat3(1,2,3,4,5,6,7,8,9)
    m2=eval(repr(m))
    self.assertTrue(geom.Equal(m, m2))

    m=geom.Mat4(1,2,3,4,5,6,7,8,9,10,11,12,13,14,15,16)
    m2=eval(repr(m))
    self.assertTrue(geom.Equal(m, m2))

  def test_data(self):
    self.assertEqual(geom.Vec2(1,2).data,[1,2])
    self.assertEqual(geom.Vec3(1,2,3).data,[1,2,3])
    self.assertEqual(geom.Vec4(1,2,3,4).data,[1,2,3,4])
    self.assertEqual(geom.Mat2(1,2,
                               3,4).data,
                     [1,2,
                      3,4])
    self.assertEqual(geom.Mat3(1,2,3,
                               4,5,6,
                               7,8,9).data,
                     [1,2,3,
                      4,5,6,
                      7,8,9])
    self.assertEqual(geom.Mat4(1,2,3,4,
                               5,6,7,8,
                               9,10,11,12,
                               13,14,15,16).data,
                     [1,2,3,4,
                      5,6,7,8,
                      9,10,11,12,
                      13,14,15,16])

  def test_boundary_checks_vec2(self): 
    v = geom.Vec2()

    v[0] = v[0]
    v[1] = v[1]
    self.assertRaises(IndexError, v.__setitem__, (-1), 1)
    self.assertRaises(IndexError, v.__setitem__, ( 2), 1)
    self.assertRaises(IndexError, v.__getitem__, (-1))
    self.assertRaises(IndexError, v.__getitem__, ( 2))

  def test_boundary_checks_vec3(self): 
    v = geom.Vec3()

    v[0] = v[0]
    v[1] = v[1]
    v[2] = v[2]
    self.assertRaises(IndexError, v.__setitem__, (-1), 1)
    self.assertRaises(IndexError, v.__setitem__, ( 3), 1)
    self.assertRaises(IndexError, v.__getitem__, (-1))
    self.assertRaises(IndexError, v.__getitem__, ( 3))

  def test_boundary_checks_vec4(self): 
    v = geom.Vec4()
    v[0] = v[0]
    v[1] = v[1]
    v[2] = v[2]
    v[3] = v[3]
    self.assertRaises(IndexError, v.__setitem__, (-1), 1)
    self.assertRaises(IndexError, v.__setitem__, ( 4), 1)
    self.assertRaises(IndexError, v.__getitem__, (-1))
    self.assertRaises(IndexError, v.__getitem__, ( 4))

  def test_boundary_checks_mat2(self): 
    m = geom.Mat2()
    m[0,0] = m[0,0]
    m[0,1] = m[0,1]
    m[1,0] = m[1,0]
    m[1,1] = m[1,1]
    self.assertRaises(IndexError, m.__setitem__, (-1,0), 1)
    self.assertRaises(IndexError, m.__setitem__, ( 2,0), 1)
    self.assertRaises(IndexError, m.__setitem__, (0,-1), 1)
    self.assertRaises(IndexError, m.__setitem__, (-1,2), 1)

    self.assertRaises(IndexError, m.__getitem__, (-1,0))
    self.assertRaises(IndexError, m.__getitem__, ( 2,0))
    self.assertRaises(IndexError, m.__getitem__, (0,-1))
    self.assertRaises(IndexError, m.__getitem__, (0,2))

  def test_boundary_checks_mat3(self): 
    m = geom.Mat3()
    m[0,0] = m[0,0]
    m[0,1] = m[0,1]
    m[0,2] = m[0,2]
    m[1,0] = m[1,0]
    m[1,1] = m[1,1]
    m[1,2] = m[1,2]
    m[2,0] = m[2,0]
    m[2,1] = m[2,1]
    m[2,2] = m[2,2]
    self.assertRaises(IndexError, m.__setitem__, (-1,0), 1)
    self.assertRaises(IndexError, m.__setitem__, ( 3,0), 1)
    self.assertRaises(IndexError, m.__setitem__, (0,-1), 1)
    self.assertRaises(IndexError, m.__setitem__, (-1,3), 1)

    self.assertRaises(IndexError, m.__getitem__, (-1,0))
    self.assertRaises(IndexError, m.__getitem__, ( 3,0))
    self.assertRaises(IndexError, m.__getitem__, (0,-1))
    self.assertRaises(IndexError, m.__getitem__, (0,3))

  def test_boundary_checks_mat4(self): 
    m = geom.Mat4()
    m[0,0] = m[0,0]
    m[0,1] = m[0,1]
    m[0,2] = m[0,2]
    m[0,3] = m[0,3]
    m[1,0] = m[1,0]
    m[1,1] = m[1,1]
    m[1,2] = m[1,2]
    m[1,3] = m[1,3]
    m[2,0] = m[2,0]
    m[2,1] = m[2,1]
    m[2,2] = m[2,2]
    m[2,3] = m[2,3]
    m[3,0] = m[3,0]
    m[3,1] = m[3,1]
    m[3,2] = m[3,2]
    m[3,3] = m[3,3]
    self.assertRaises(IndexError, m.__setitem__, (-1,0), 1)
    self.assertRaises(IndexError, m.__setitem__, ( 4,0), 1)
    self.assertRaises(IndexError, m.__setitem__, (0,-1), 1)
    self.assertRaises(IndexError, m.__setitem__, (-1,4), 1)

    self.assertRaises(IndexError, m.__getitem__, (-1,0))
    self.assertRaises(IndexError, m.__getitem__, ( 4,0))
    self.assertRaises(IndexError, m.__getitem__, (0,-1))
    self.assertRaises(IndexError, m.__getitem__, (0,4))

if __name__== '__main__':
  from ost import testutils
  testutils.RunTests()




import math,random
from ost import img

# remove all objects from scene, just in case
scene.RemoveAll()

vmax=-10000.0
vmin=+10000.0
mh=img.CreateMap(img.Size(32,32,32))
for p in img.ExtentIterator(mh.GetExtent()):
  val=5*math.sin(0.4*math.sqrt(p[0]*p[0]+p[1]*p[1]))+7*math.cos(0.6*math.sqrt(p[2]*p[2]+p[1]*p[1]))
  mh.SetReal(p,val)
  vmin=min(vmin,val)
  vmax=max(vmax,val)
print(vmin, vmax)
for p in img.ExtentIterator(mh.GetExtent()):
  mh.SetReal(p,(mh.GetReal(p)-vmin)/(vmax-vmin))

pl = gfx.PrimList("box")
edge_list=[[[0,0,0],[0,1,0]],
           [[0,1,0],[1,1,0]],
           [[1,1,0],[1,0,0]],
           [[1,0,0],[0,0,0]],
           [[0,0,0],[0,0,1]],
           [[0,1,0],[0,1,1]],
           [[1,1,0],[1,1,1]],
           [[1,0,0],[1,0,1]],
           [[0,0,1],[0,1,1]],
           [[0,1,1],[1,1,1]],
           [[1,1,1],[1,0,1]],
           [[1,0,1],[0,0,1]]]

corners = [mh.IndexToCoord(mh.GetExtent().GetStart()),mh.IndexToCoord(mh.GetExtent().GetEnd())]

for e in edge_list:
    pl.AddLine(geom.Vec3(corners[e[0][0]][0],
                    corners[e[0][1]][1],
                    corners[e[0][2]][2]),
               geom.Vec3(corners[e[1][0]][0],
                    corners[e[1][1]][1],
                    corners[e[1][2]][2]),
               gfx.Color(1,1,0))

scene.Add(pl)
scene.SetCenter(pl.GetCenter())

go1=gfx.MapIso("iso", mh,0.5)
go1.SetLineWidth(1.5)
scene.Add(go1)
scene.SetCenter(go1.GetCenter())


go2 = gfx.MapSlab("slab",mh,geom.Plane(go1.GetCenter(),geom.Vec3(0.0,0.0,1.0)))
scene.Add(go2)

go2.ColorBy('HEAT_MAP',0.2,0.8)

print('Demo 4: Projecting the density of a map onto a plane...') 



"""
This script illustrates how to implement a custom tool for interacting with the
3D scene.
"""
from ost import gui


class ColorTool(gui.Tool):
  def __init__(self):
    gui.Tool.__init__(self, "Color Tool")   
    self.tool_options.AddOption(gui.ToolOptionFloat("r", "Red", 0.5, 0.0, 1.0))
    self.tool_options.AddOption(gui.ToolOptionFloat("g", "Green", 0.5, 0.0, 1.0))
    self.tool_options.AddOption(gui.ToolOptionFloat("b", "Blue", 0.5, 0.0, 1.0))
    apply_to=gui.ToolOptionEnum("apply_to", "Apply To")
    apply_to.Add("Atom", 0)
    apply_to.Add("Residue", 1)
    apply_to.Add("Chain", 2)
    apply_to.SetIndex(0)
    self.tool_options.AddOption(apply_to)

  def GetColor(self):
    return gfx.Color(self.tool_options.GetOption('r').value,
                     self.tool_options.GetOption('g').value,
                     self.tool_options.GetOption('b').value)
                     
  def CanOperateOn(self, obj):
    return True
  def GetIconPath(self):
    return ''
  def GetAtomsToColor(self, atom):
    apply_to=self.tool_options.GetOption('apply_to').value
    if apply_to==0:
      return [atom]
    if apply_to==1:
      return atom.residue.atoms
    if apply_to==2:
      l=[]
      for r in atom.residue.chain.residues:
        for a in r.atoms:
          l.append(a)
      return l

  def Click(self, event):
    obj, atom=gfx.PickAtom(scene, event.pos.x(), event.pos.y())
    if atom.IsValid():
      atoms=self.GetAtomsToColor(atom)
      color=self.GetColor()
      for atom in atoms:
        obj.SetColorForAtom(color, atom.handle)

color_tool=ColorTool()

gui.ToolManager.Instance().AddTool(color_tool)

e=io.LoadEntity(len(sys.argv)>1 and sys.argv[1] or '../entity/fragment.pdb')
g=gfx.Entity('e', e)
g.SetRenderMode(gfx.RenderMode.CUSTOM)
scene.Add(g)
scene.center=g.center



lines=gfx.PrimList("lines")

g=gfx.Gradient()
g.SetColorAt(0, gfx.Color(1.0, 0.0, 0.0, 1.0))
g.SetColorAt(0.5, gfx.Color(0.0, 1.0, 0.0, 1.0))
g.SetColorAt(1.0, gfx.Color(0.0, 0.0, 1.0, 1.0))

for i in range(1000):
  c=g.GetColorAt(0.001*i)
  lines.AddLine(geom.Vec3(i*0.05, 0.0, -1), geom.Vec3(i*0.05, 0.0, 1), c)

scene.Add(lines)
scene.center=geom.Vec3(25, 0, 0)



import unittest
import subprocess
import ost
from ost import *

class TestMMCifInfo(unittest.TestCase):
  def setUp(self):
    pass

  def test_mmcifinfo_citation(self):
    c = io.MMCifInfoCitation()
    # test ID setting/ getting
    c.SetID('ID')
    self.assertEqual(c.GetID(), 'ID')
    # test CAS setting/ getting
    c.SetCAS('FOO')
    self.assertEqual(c.GetCAS(), 'FOO')
    # test ISBN setting/ getting
    c.SetISBN('0-0-0-0-0-0')
    self.assertEqual(c.GetISBN(), '0-0-0-0-0-0')
    # test published_in setting/ getting
    c.SetPublishedIn('Best Book Ever')
    self.assertEqual(c.GetPublishedIn(), 'Best Book Ever')
    # test volume setting/ getting
    c.SetVolume('3')
    self.assertEqual(c.GetVolume(), '3')
    # test page setting/ getting
    c.SetPageFirst('1')
    self.assertEqual(c.GetPageFirst(), '1')
    c.SetPageLast('10')
    self.assertEqual(c.GetPageLast(), '10')
    # test doi setting/ getting
    c.SetDOI('HERE')
    self.assertEqual(c.GetDOI(), 'HERE')
    # test PubMed setting/ getting
    c.SetPubMed(815)
    self.assertEqual(c.GetPubMed(), 815)
    # test year setting/ getting
    c.SetYear(815)
    self.assertEqual(c.GetYear(), 815)
    # test title setting/ getting
    c.SetTitle('Foo')
    self.assertEqual(c.GetTitle(), 'Foo')
    # test book_publisher set & get
    c.SetBookPublisher("Hugo")
    self.assertEqual(c.GetBookPublisher(), "Hugo")
    # test book_publisher_city set & get
    c.SetBookPublisherCity("Basel")
    self.assertEqual(c.book_publisher_city, "Basel")
    # test citation type
    self.assertTrue(c.IsCitationTypeUnknown())
    self.assertEqual(c.citation_type, io.MMCifInfoCType.Unknown)
    # test auhtors setting/ getting
    s = ost.StringList()
    s.append('Foo')
    c.SetAuthorList(s)
    s2 = c.GetAuthorList()
    self.assertEqual(s2[0], 'Foo')

    i = io.MMCifInfo()
    i.SetMethod('Deep-Fry')
    i.SetResolution(2.0)
    i.AddCitation(c)
    s.append('Bar')
    i.AddAuthorsToCitation('ID', s)

    cl = i.GetCitations()
    self.assertEqual(len(cl), 1)
    al = cl[0].GetAuthorList()
    self.assertEqual(len(al), 2)
    self.assertEqual(al[0], 'Foo')
    self.assertEqual(al[1], 'Bar')

    self.assertEqual(i.GetMethod(), 'Deep-Fry')
    self.assertEqual(i.GetResolution(), 2.0)


  def test_mmcifinfo_biounit(self):
    b = io.MMCifInfoBioUnit()
    b.SetDetails('Details')
    b.SetMethodDetails('MethodDetails')
    self.assertEqual(b.GetDetails(), 'Details')
    self.assertEqual(b.GetMethodDetails(), 'MethodDetails')
    b.method_details = 'AttrMethodDetails'
    self.assertEqual(b.method_details, 'AttrMethodDetails')
    b.AddChain('A')
    b.AddChain('B')
    cl = b.GetChainList()
    il = b.GetChainIntervalList()
    self.assertEqual(cl[0], 'A')
    self.assertEqual(il[0][0], 0)
    self.assertEqual(il[0][1], 2)
    s = ost.StringList()
    s.append('B')
    s.append('C')
    s.append('D')
    b.SetChainList(s)
    cl = b.GetChainList()
    il = b.GetChainIntervalList()
    self.assertEqual(il[0][0], 0)
    self.assertEqual(il[0][1], 3)
    self.assertEqual(cl[0], 'B')
    self.assertEqual(cl[1], 'C')
    self.assertEqual(cl[2], 'D')

    i = io.MMCifInfo()
    i.AddBioUnit(b)
    i.AddBioUnit(b)
    b.SetID("2")
    i.AddBioUnit(b)

    bl = i.GetBioUnits()
    il = bl[0].GetChainIntervalList()
    self.assertEqual(il[0][0], 0)
    self.assertEqual(il[0][1], 3)
    self.assertEqual(il[1][0], 3)
    self.assertEqual(il[1][1], 6)
    self.assertEqual(len(bl), 2)


  def test_mmcifinfo_transoperation(self):
    o = io.MMCifInfoTransOp()
    o.SetID("1")
    self.assertEqual(o.GetID(), '1')
    o.SetType("identity operation")
    self.assertEqual(o.GetType(), 'identity operation')
    o.SetVector(1.0, 2.0, 3.0)
    self.assertEqual(o.GetVector().x, 1.0)
    self.assertEqual(o.GetVector().y, 2.0)
    self.assertEqual(o.GetVector().z, 3.0)
    o.SetMatrix(1, 2, 3, 4, 5, 6, 7, 8, 9)
    self.assertEqual(geom.Equal(o.GetMatrix(),
                                 geom.Mat3(1, 2, 3, 4, 5, 6, 7, 8, 9)), True)

    i = io.MMCifInfo()
    i.AddOperation(o)
    ol = i.GetOperations()
    self.assertEqual(ol[0].GetID(), '1')

    b = io.MMCifInfoBioUnit()
    b.AddOperations(ol)
    oll = b.GetOperations()
    self.assertEqual(oll[0][0].GetID(), '1')
    tr_ol = b.GetOperationsIntervalList()
    self.assertEqual(len(tr_ol), 1)
    self.assertEqual(tr_ol[0][0], 0)
    self.assertEqual(tr_ol[0][1], 1)

  def test_mmcifinfo_biounit_pdbize_with_multiple_transforms(self):
    ent, seqres, info = io.LoadMMCIF("testfiles/mmcif/multiple_transforms.cif.gz",
                                     seqres=True,
                                     info=True)
    pdb_ent = info.GetBioUnits()[0].PDBize(ent)
    chains = pdb_ent.chains
    self.assertEqual(''.join([c.name for c in chains]), 
                      'A_-BCD')
    ligand_chain = chains[1]
    ligand_residues = ligand_chain.residues
    self.assertEqual([r.number for r in ligand_residues],
                      [mol.ResNum(1), mol.ResNum(2), mol.ResNum(3), mol.ResNum(4)])

  def test_mmcifinfo_biounit_pdbize(self):
    ent, seqres, info = io.LoadMMCIF("testfiles/mmcif/3T6C.cif.gz",
                                     seqres=True,
                                     info=True)
    pdb_ent = info.GetBioUnits()[0].PDBize(ent)
    pdb_seqres_ent = info.GetBioUnits()[0].PDBize(ent, seqres)

    self.assertEqual(''.join([c.name for c in pdb_ent.chains]),
                      'AB_-CDEFGH')
    self.assertEqual([c.residue_count for c in pdb_ent.chains],
                      [415, 414, 64, 3816, 415, 414, 415, 414, 415, 414])
    self.assertEqual([c.atom_count for c in pdb_ent.chains],
                      [3231, 3223, 268, 3816, 3231, 3223, 3231, 3223, 3231, 3223])
    self.assertEqual([c.GetBondCount() for c in pdb_ent.chains],
                      [3311, 3303, 204, 0, 3311, 3303, 3311, 3303, 3311, 3303])

    self.assertEqual(''.join([c.name for c in pdb_seqres_ent.chains]),
                      ''.join([c.name for c in pdb_ent.chains]))
    self.assertEqual([c.residue_count for c in pdb_seqres_ent.chains],
                      [c.residue_count for c in pdb_ent.chains])
    self.assertEqual([c.atom_count for c in pdb_seqres_ent.chains],
                      [c.atom_count for c in pdb_ent.chains])
    self.assertEqual([c.GetBondCount() for c in pdb_seqres_ent.chains],
                      [c.GetBondCount() for c in pdb_ent.chains])

  def test_mmcifinfo_biounit_pdbize_transformation(self):
    ent, seqres, info = io.LoadMMCIF("testfiles/mmcif/3hqv.cif.gz",
                                     seqres=True,
                                     info=True)
    pdb_ent, t = info.GetBioUnits()[0].PDBize(ent, transformation=True)
    self.assertAlmostEqual(pdb_ent.GetCenterOfAtoms()[0], -915.759, 1)
    self.assertAlmostEqual(pdb_ent.GetCenterOfAtoms()[1], -952.345, 1)
    self.assertAlmostEqual(pdb_ent.GetCenterOfAtoms()[2], 3221.75, 1)
    self.assertEqual(geom.Equal(t,
                                 geom.Mat4(1,0,0,-920.462,
                                           0,1,0,-966.654,
                                           0,0,1,1703,
                                           0,0,0,1),
                                 epsilon=0.01), True)

  def test_mmcifinfo_structdetails(self):
    d = io.MMCifInfoStructDetails()

    d.SetEntryID('1BAR')
    d.SetTitle('A Title')
    d.SetCASPFlag('N')
    d.SetDescriptor('FooBar')
    d.SetMass(1.0)
    d.SetMassMethod('Good Guess')
    d.SetModelDetails('Created with SwissModel')
    d.SetModelTypeDetails('Average')
    self.assertEqual(d.GetEntryID(), '1BAR')
    self.assertEqual(d.GetTitle(), 'A Title')
    self.assertEqual(d.GetCASPFlag(), 'N')
    self.assertEqual(d.GetDescriptor(), 'FooBar')
    self.assertEqual(d.GetMass(), 1.0)
    self.assertEqual(d.GetMassMethod(), 'Good Guess')
    self.assertEqual(d.GetModelDetails(), 'Created with SwissModel')  
    self.assertEqual(d.GetModelTypeDetails(), 'Average') 

    i = io.MMCifInfo()
    i.SetStructDetails(d)
    self.assertEqual(i.GetStructDetails().GetEntryID(), '1BAR')
    self.assertEqual(i.GetStructDetails().GetTitle(), 'A Title')
    self.assertEqual(i.GetStructDetails().GetCASPFlag(), 'N')
    self.assertEqual(i.GetStructDetails().GetDescriptor(), 'FooBar')
    self.assertEqual(i.GetStructDetails().GetMass(), 1.0)
    self.assertEqual(i.GetStructDetails().GetMassMethod(), 'Good Guess')
    self.assertEqual(i.GetStructDetails().GetModelDetails(),
                      'Created with SwissModel')
    self.assertEqual(i.GetStructDetails().GetModelTypeDetails(), 'Average')

  def test_mmcifinfo_obsolete(self):
    obs = io.MMCifInfoObsolete()
    obs.SetDate('2011-08-31')
    obs.SetID('SPRSDE')
    obs.SetPDBID('1FOO')
    obs.SetReplacedPDBID('2BAR')
    self.assertEqual(obs.GetDate(), '2011-08-31')
    self.assertEqual(obs.GetID(), 'Supersede')
    self.assertEqual(obs.GetPDBID(), '1FOO')
    self.assertEqual(obs.GetReplacedPDBID(), '2BAR')

    i = io.MMCifInfo()
    obs.id = 'OBSLTE'
    i.SetObsoleteInfo(obs)
    self.assertEqual(i.GetObsoleteInfo().GetDate(), '2011-08-31')
    self.assertEqual(i.GetObsoleteInfo().GetID(), 'Obsolete')
    self.assertEqual(i.GetObsoleteInfo().GetPDBID(), '1FOO')
    self.assertEqual(i.GetObsoleteInfo().GetReplacedPDBID(), '2BAR')

  def test_remote_loading(self):

    if subprocess.call(['ping google.com -c 1'], shell=True,
                       stdout=subprocess.PIPE, stderr=subprocess.PIPE) != 0:
      print("No internet connection (or wrong OS) to test remote loading in "
            "io.LoadMMCIF: ignoring unit test")
      return

    # let's hope that crambin stays the same forever
    crambin_pdb = io.LoadMMCIF('1crn', remote=True)
    self.assertEqual(len(crambin_pdb.residues), 46)
    self.assertEqual(len(crambin_pdb.atoms), 327)

  def test_mmcifinfo_entitybranch(self):
    # test MMCifInfoEntityBranchLink
    eh = mol.CreateEntity()
    editor = eh.EditXCS();
    ch = editor.InsertChain("A");
    res1 = editor.AppendResidue(ch, "BMA");
    res2 = editor.AppendResidue(ch, "MAN");
    atom1 = editor.InsertAtom(res2, "C1", geom.Vec3());
    atom2 = editor.InsertAtom(res1, "O3", geom.Vec3());
    branch = io.MMCifInfoEntityBranchLink(atom1, atom2, 1)
    self.assertEqual(branch.atom1.qualified_name, "A.MAN2.C1")
    self.assertEqual(branch.bond_order, 1)

    branch.ConnectBranchLink(editor)
    self.assertEqual(atom2.GetBondPartners()[0].qualified_name, "A.MAN2.C1")

    # test entity_branches_
    ch = editor.InsertChain("B");
    res1 = editor.AppendResidue(ch, "NAG");
    res2 = editor.AppendResidue(ch, "NAG");
    atom3 = editor.InsertAtom(res2, "C1", geom.Vec3());
    atom4 = editor.InsertAtom(res1, "O4", geom.Vec3());
    info = io.MMCifInfo()
    info.AddEntityBranchLink("A", atom1, atom2, 1)
    info.AddEntityBranchLink(ch.name, atom3, atom4, 1)

    blinks = info.GetEntityBranchLinks()
    self.assertEqual(blinks[0].GetAtom1().qualified_name, "A.MAN2.C1")
    self.assertEqual(blinks[0].atom2.qualified_name, "A.BMA1.O3")
    self.assertEqual(blinks[0].GetBondOrder(), 1)
    self.assertEqual(blinks[1].atom1.qualified_name, "B.NAG2.C1")
    self.assertEqual(blinks[1].GetAtom2().qualified_name, "B.NAG1.O4")
    self.assertEqual(blinks[1].GetBondOrder(), 1)

    info.ConnectBranchLinks()
    self.assertEqual(atom4.GetBondPartners()[0].qualified_name, "B.NAG2.C1")

    chain_names = info.GetEntityBranchChainNames()
    self.assertEqual(chain_names, ['A', 'B'])
    chains = info.GetEntityBranchChains()
    self.assertEqual(chains[0].name, 'A')
    self.assertEqual(chains[1].name, 'B')

    blinks = info.GetEntityBranchByChain('B')
    self.assertEqual(len(blinks), 1)
    self.assertEqual(blinks[0].atom1.qualified_name, "B.NAG2.C1")
    blinks = info.GetEntityBranchByChain('C')
    self.assertEqual(len(blinks), 0)

if __name__== '__main__':
  from ost import testutils
  testutils.RunTests()





#------------------------------------------------------------------------------
# This file is part of the OpenStructure project <www.openstructure.org>
#
# Copyright (C) 2008-2020 by the OpenStructure authors
#
# This library is free software; you can redistribute it and/or modify it under
# the terms of the GNU Lesser General Public License as published by the Free
# Software Foundation; either version 3.0 of the License, or (at your option)
# any later version.
# This library is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
# FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public License for more
# details.
#
# You should have received a copy of the GNU Lesser General Public License
# along with this library; if not, write to the Free Software Foundation, Inc.,
# 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
#------------------------------------------------------------------------------
# -*- coding: utf-8 -*-

from ost import gui
from ost import gfx
from PyQt5 import QtCore, QtWidgets
from .render_mode_widget import RenderModeWidget

#Simple Render Options
class LineTraceWidget(RenderModeWidget):
  def __init__(self, parent=None):
    RenderModeWidget.__init__(self, parent)
    
    #Title
    self.text_ = "Fast Trace"
    
    #Set Render Mode
    self.mode_ = gfx.RenderMode.LINE_TRACE
    
    #Defaults
    min_line_width = 0.01
    max_line_width = 20
        
    #Create Ui elements
    self.radius_spinbox_ = QtWidgets.QDoubleSpinBox()
    self.radius_spinbox_.setRange(min_line_width, max_line_width)
    self.radius_spinbox_.setDecimals(2)
    self.radius_spinbox_.setSingleStep(0.1)
    
    self.aa_rendering_cb_ = QtWidgets.QCheckBox()
    
    sline_label = QtWidgets.QLabel("Trace Settings")
    font = sline_label.font()
    font.setBold(True)
    
    radius_label = QtWidgets.QLabel("Line Width")
    aa_label = QtWidgets.QLabel("AA-Lines")
    grid = QtWidgets.QGridLayout()
    grid.addWidget(sline_label, 0, 0, 1, 1)
    grid.addWidget(radius_label, 1, 0, 1, 3)
    grid.addWidget(self.radius_spinbox_, 1, 2, 1, 1)
    grid.addWidget(aa_label, 2, 0, 1, 3)
    grid.addWidget(self.aa_rendering_cb_, 2, 2, 1, 1)
    grid.setRowStretch(3,1)
    self.setLayout(grid)

    self.radius_spinbox_.valueChanged.connect(self.UpdateLineWidth)
    self.aa_rendering_cb_.stateChanged.connect(self.UpdateAA)

    self.setMinimumSize(250,90)
    
  def UpdateAA(self, value):
    self.GetOptions().SetAALines(value)
    self.ApplyOptions()
    
  def UpdateLineWidth(self, value):
    self.GetOptions().SetLineWidth(value)
    self.ApplyOptions()
    
  def UpdateGui(self,options):
    self.aa_rendering_cb_.setChecked(options.GetAALines())    
    self.radius_spinbox_.setValue(options.GetLineWidth())

  def GetText(self):
    return self.text_

  def GetRenderMode(self):
    return self.mode_



import unittest
import os
import ost
from ost import seq, io, mol

class TestAlignmentFromChainView(unittest.TestCase):

  def testAlignment(self):

    prot = io.LoadPDB(os.path.join("testfiles", "dummy.pdb"))
    view_resnums = [2,3,4,7,8]
    query = "rnum=" + ','.join([str(item) for item in view_resnums])
    prot_view = prot.Select(query)

    exp_handle_seq = ''.join([r.one_letter_code for r in prot.residues])

    exp_view_seq = ['-'] * len(prot.residues)
    for rnum in view_resnums:
      exp_view_seq[rnum-1] = prot.residues[rnum-1].one_letter_code
    exp_view_seq = ''.join(exp_view_seq)

    aln = seq.alg.AlignmentFromChainView(prot_view.chains[0], 
                                         handle_seq_name='custom_handle_name',
                                         view_seq_name='custom_view_name')

    handle_seq = aln.GetSequence(0)
    view_seq = aln.GetSequence(1)

    self.assertEqual(handle_seq.GetName(), 'custom_handle_name')
    self.assertEqual(view_seq.GetName(), 'custom_view_name')
    self.assertEqual(exp_handle_seq, handle_seq.GetString())
    self.assertEqual(exp_view_seq, view_seq.GetString())

if __name__ == "__main__":
  from ost import testutils
  testutils.RunTests()



import ost as _ost
import ost.geom as _geom

"""
Module written by Niklaus Johner (niklaus.johner@a3.epfl.ch) 2012

This module is a flexible rewrite of HBPlus, allowing to calculate hbond
conservation between different structures or over a trajectory. 
It uses customizable dictionaries to define donors and acceptors and can
account for equivalent HBonds such as involving for example either oxygen atom
of an Aspartic Acid.
"""
__all__=('HBondDonor','HBondAcceptor','BuildCHARMMHBondDonorAcceptorDict','AreHBonded','GetHbondDonorAcceptorList',\
         'GetHbondListFromDonorAcceptorLists','GetHbondListFromView','GetHbondListFromTraj','GetHbondListBetweenViews'\
         'CalculateHBondScore','AnalyzeHBondScore')
         
         
class HBondableAtoms:
	def __init__(self,donors=[],acceptors=[]):
		self.donors=donors
		self.acceptors=acceptors
    
def BuildCHARMMHBondDonorAcceptorDict():
  hb_da_dict={}
  bb_donors=[['N','HN']]
  bb_acceptors=[['O',['C']]]
  
  #hydrophobic
  hb_da_dict['ALA']=HBondableAtoms(bb_donors,bb_acceptors)
  hb_da_dict['GLY']=HBondableAtoms(bb_donors,bb_acceptors)
  hb_da_dict['ILE']=HBondableAtoms(bb_donors,bb_acceptors)
  hb_da_dict['LEU']=HBondableAtoms(bb_donors,bb_acceptors)
  hb_da_dict['PHE']=HBondableAtoms(bb_donors,bb_acceptors)
  hb_da_dict['MET']=HBondableAtoms(bb_donors,bb_acceptors)
  hb_da_dict['VAL']=HBondableAtoms(bb_donors,bb_acceptors)
  #special case
  hb_da_dict['PRO']=HBondableAtoms([],bb_acceptors)
  
  #sidechain donors
  ne=[['NE','HE']]
  cz=[['NH1','HH11'],['NH1','HH12'],['NH2','HH21'],['NH2','HH22']]
  hb_da_dict['ARG']=HBondableAtoms(bb_donors+ne+cz,bb_acceptors)
  nz=[['NZ','HZ1'],['NZ','HZ2'],['NZ','HZ3']]
  hb_da_dict['LYS']=HBondableAtoms(bb_donors+nz,bb_acceptors)
  ne1=[['NE1','HE1']]
  hb_da_dict['TRP']=HBondableAtoms(bb_donors+ne1,bb_acceptors)
  sg=[['SG','HG1']]
  hb_da_dict['CYS']=HBondableAtoms(bb_donors+sg,bb_acceptors)
  
  #sidechain acceptors
  od12=[['OD1',['CG']],['OD2',['CG']]]
  hb_da_dict['ASP']=HBondableAtoms(bb_donors,bb_acceptors+od12)
  oe12=[['OE1',['CD']],['OE2',['CD']]]
  hb_da_dict['GLU']=HBondableAtoms(bb_donors,bb_acceptors+oe12)
  
  #sidechain donor and acceptor
  od1=[['OD1',['CG']]]
  nd2=[['ND2','HD21'],['ND2','HD22']]
  hb_da_dict['ASN']=HBondableAtoms(bb_donors+nd2,bb_acceptors+od1)
  ne2=[['NE2','HE21'],['NE2','HE22']]
  oe1=[['OE1',['CD']]]
  hb_da_dict['GLN']=HBondableAtoms(bb_donors+ne2,bb_acceptors+oe1)
  og_d=[['OG','HG1']]
  og_a=[['OG',['CB']]]
  hb_da_dict['SER']=HBondableAtoms(bb_donors+og_d,bb_acceptors+og_a)
  og1_d=[['OG1','HG1']]
  og1_a=[['OG1',['CB']]]
  hb_da_dict['THR']=HBondableAtoms(bb_donors+og1_d,bb_acceptors+og1_a)
  oh_d=[['OH','HH']]
  oh_a=[['OH',['CZ']]]
  hb_da_dict['TYR']=HBondableAtoms(bb_donors+oh_d,bb_acceptors+oh_a)
  #histidine
  nd1_d=[['ND1','HD1']]
  ne2_a=[['NE2',['CD2','CE1']]]
  hb_da_dict['HSD']=HBondableAtoms(bb_donors+nd1_d,bb_acceptors+ne2_a)
  ne2_d=[['NE2','HE2']]
  nd1_a=[['ND1',['CG','CE1']]]
  hb_da_dict['HSE']=HBondableAtoms(bb_donors+ne2_d,bb_acceptors+nd1_a)
  hb_da_dict['HSP']=HBondableAtoms(bb_donors+nd1_d+ne2_d,bb_acceptors)
  #non-standard protonation state:
  oe12=[['OE1',['CD']],['OE2',['CD']]]
  oe2=[['OE2','HE2']]
  hb_da_dict['GLUP']=HBondableAtoms(bb_donors+oe2,bb_acceptors+oe12)
  od12=[['OD1',['CG']],['OD2',['CG']]]
  od2=[['OD2','HD2']]
  hb_da_dict['ASPP']=HBondableAtoms(bb_donors+od2,bb_acceptors+od12)
  return hb_da_dict

def BuildCHARMMHBondDonorEquivalenceDict():
  donor_swap_dict={}
  donor_swap_dict['ARG']=[[['NH1','HH11'],['NH1','HH12'],['NH2','HH21'],['NH2','HH22']]]
  donor_swap_dict['ASN']=[[['ND2','HD21'],['ND2','HD22']]]
  donor_swap_dict['GLN']=[[['NE2','HE21'],['NE2','HE22']]]
  donor_swap_dict['LYS']=[[['NZ','HZ1'],['NZ','HZ2'],['NZ','HZ3']]]
  return donor_swap_dict

def BuildCHARMMHBondAcceptorEquivalenceDict():
  acceptor_swap_dict={}
  acceptor_swap_dict['ASP']=[[['OD1',['CG']],['OD2',['CG']]]]
  acceptor_swap_dict['GLU']=[[['OE1',['CD']],['OE2',['CD']]]]
  return acceptor_swap_dict

def ListEquivalentDonors(donor,donor_swap_dict):
  if not donor.heavy_atom.residue.name in donor_swap_dict:return [donor]
  donor_list=[donor]
  donor_atom_names=[donor.heavy_atom.name,donor.hydrogen.name]
  res=donor.heavy_atom.residue
  for equivalence_list in donor_swap_dict[donor.heavy_atom.residue.name]:
    if not donor_atom_names in equivalence_list:continue
    for atom_names in equivalence_list:
      if atom_names==donor_atom_names:continue
      else:donor_list.append(HBondDonor.FromResidue(res,atom_names[0],atom_names[1]))
  return donor_list

def ListEquivalentAcceptors(acceptor,acceptor_swap_dict):
  if not acceptor.atom.residue.name in acceptor_swap_dict:return [acceptor]
  acceptor_list=[acceptor]
  acceptor_atom_names=[acceptor.atom.name,[a.name for a in acceptor.antecedent_list]]
  res=acceptor.atom.residue
  for equivalence_list in acceptor_swap_dict[acceptor.atom.residue.name]:
    if not acceptor_atom_names in equivalence_list:continue
    for atom_names in equivalence_list:
      if atom_names==acceptor_atom_names:continue
      else:acceptor_list.append(HBondAcceptor.FromResidue(res,atom_names[0],atom_names[1]))
  return acceptor_list

  
class HBondDonor:
  def __init__(self,donor,hydrogen):
    self.heavy_atom=donor
    self.hydrogen=hydrogen
  def __eq__(self,hbd):
    if not isinstance(hbd,HBondDonor):return False
    else:return self.heavy_atom==hbd.heavy_atom and self.hydrogen==hbd.hydrogen
  def __hash__(self):
    return hash((self.heavy_atom,self.hydrogen))
  @classmethod
  def FromResidue(cls,res,donor_name,hydrogen_name,verbose=True):
    _donor=res.FindAtom(donor_name).handle
    _hydrogen=res.FindAtom(hydrogen_name).handle
    if not _donor.IsValid():
      if verbose:print('Could not find '+donor_name+' in residue '+str(res))
      return
    if not _hydrogen.IsValid():
      if verbose:print('Could not find '+hydrogen_name+' in residue '+str(res))
      return
    return cls(_donor,_hydrogen)
  
  def IsHBondedTo(self,acceptor):
    return AreHBonded(self,acceptor)


class HBondAcceptor:
  def __init__(self,acceptor,antecedent_list):
    self.atom=acceptor
    self.antecedent_list=antecedent_list
  def __eq__(self,hba):
    if not isinstance(hba,HBondAcceptor):return False
    else:return self.atom==hba.atom
  def __hash__(self):
    return hash(self.atom)
  @classmethod
  def FromResidue(cls,res,acceptor_name,antecedent_name_list,verbose=True):
    _acceptor=res.FindAtom(acceptor_name).handle
    _antecedent_list=_ost.mol.AtomHandleList([res.FindAtom(name).handle for name in antecedent_name_list])
    if not _acceptor.IsValid():
      if verbose:print('Could not find '+acceptor_name+' in residue '+str(res))
      return
    for i,a in enumerate(_antecedent_list):
      if not a.IsValid():
        if verbose:print('Could not find '+antecedent_name_list[i]+' in residue '+str(res))
        return
    return cls(_acceptor,_antecedent_list)
  
  def IsHBondedTo(self,donor):
    return AreHBonded(donor,self)

class HBond:
  def __init__(self,donor,acceptor):
    self.donor=donor
    self.acceptor=acceptor
  def __eq__(self,hb):
    if not isinstance(hb,HBond):return False
    else:return self.acceptor==hb.acceptor and self.donor==hb.donor
  def __hash__(self):
    return hash((self.donor,self.acceptor))
  def IsFormed(self):return AreHBonded(self.donor,self.acceptor)
  def DonorAcceptorDistance(self):return _geom.Distance(self.donor.heavy_atom.pos,self.acceptor.atom.pos)
  def HydrogenAcceptorDistance(self):return _geom.Distance(self.donor.hydrogen.pos,self.acceptor.atom.pos)
  def DonorHydrogenAcceptorAngle(self):
    dp=self.donor.heavy_atom.pos;ap=self.acceptor.atom.pos;hp=self.donor.hydrogen.pos
    return _geom.Angle(dp-hp,ap-hp)
  def DonorAcceptorAcceptorAntecedentAngle(self):
    dp=self.donor.heavy_atom.pos;ap=self.acceptor.atom.pos
    a=min([_geom.Angle(aa.pos-ap,dp-ap) for aa in self.acceptor.antecedent_list])
    return a
  def HydrogenAcceptorAcceptorAntecedentAngle(self):
    hp=self.donor.hydrogen.pos;ap=self.acceptor.atom.pos
    a=min([_geom.Angle(aa.pos-ap,hp-ap) for aa in acceptor.antecedent_list])
    return a

    
def AreHBonded(donor,acceptor,da_dist=3.9,ha_dist=2.5,dha_angle=1.57,daaa_angle=1.57,haaa_angle=1.57):
  """
  determines if a donor/acceptor pair is hydrogen bonded or not
  """
  dp=donor.heavy_atom.pos
  ap=acceptor.atom.pos
  if _geom.Distance(dp,ap)>da_dist:return False
  hp=donor.hydrogen.pos
  if _geom.Distance(hp,ap)>ha_dist:return False
  if _geom.Angle(dp-hp,ap-hp)<dha_angle:return False
  a=min([_geom.Angle(aa.pos-ap,dp-ap) for aa in acceptor.antecedent_list])
  if a<daaa_angle:return False
  a=min([_geom.Angle(aa.pos-ap,hp-ap) for aa in acceptor.antecedent_list])
  if a<haaa_angle:return False
  return True

def GetHbondDonorAcceptorList(eh,hbond_donor_acceptor_dict={},verbose=True):
  """
  returns a list of hydrogen-bond donors and acceptors from an Entity or EntityView.
  It relies on atom names to determine the list of H-bond donors and acceptors.
  These names are given in a dictionary, which defaults to CHARMM.
  """
  if not hbond_donor_acceptor_dict:
    print('Using default CHARMM atom namings')
    hbond_donor_acceptor_dict=BuildCHARMMHBondDonorAcceptorDict()
  donor_list=[]
  acceptor_list=[]
  for r in eh.residues:
    if not r.name in hbond_donor_acceptor_dict:
      print('donors and acceptors for',r,'are not defined in the dictionary and will not be included')
      continue
    res_da_dict=hbond_donor_acceptor_dict[r.name]
    for acceptor in res_da_dict.acceptors:
      a=HBondAcceptor.FromResidue(r,acceptor[0],acceptor[1],verbose)
      if not a==None:acceptor_list.append(a)
    for donor in res_da_dict.donors:
      d=HBondDonor.FromResidue(r,donor[0],donor[1],verbose)
      if not d==None:donor_list.append(d)
  return [donor_list,acceptor_list]  
  
  
def GetHbondListFromDonorAcceptorLists(donor_list,acceptor_list):
  """
  return a list of hydrogen bonds between donors and acceptors from
  a list of donors and a list of acceptors.
  """
  hbond_list=[]
  for donor in donor_list:
    for acceptor in acceptor_list:
      if AreHBonded(donor,acceptor):hbond_list.append(HBond(donor,acceptor))
  return hbond_list

def GetHbondListFromView(eh,hbond_donor_acceptor_dict={},verbose=True):
  """
  return a list of hydrogen bonds from an Entity or EntityView.
  if no dictionary for the hbond donors and acceptors is specified
  it will use the standard CHARMM names to determine them
  """
  [donor_list,acceptor_list]=GetHbondDonorAcceptorList(eh,hbond_donor_acceptor_dict,verbose)
  hbond_list=[]
  for donor in donor_list:
    for acceptor in acceptor_list:
      if AreHBonded(donor,acceptor):hbond_list.append(HBond(donor,acceptor))
  return hbond_list

def GetHbondListFromTraj(t,eh,cutoff=0.7,stride=1,swap=False,donor_swap_dict={},acceptor_swap_dict={},hbond_donor_acceptor_dict={},verbose=True):
  """
  return a list of hydrogen bonds from an Entity or EntityView that are
  present in a fraction of the frames larger than *cutoff*.
  if no dictionary for the hbond donors and acceptors is specified
  it will use the standard CHARMM names to determine them
  """
  if swap:
    if not donor_swap_dict:
      print('use of standard CHARMM HBond donor swap dictionary')
      donor_swap_dict=BuildCHARMMHBondDonorEquivalenceDict()
    if not acceptor_swap_dict:
      print('use of standard CHARMM HBond acceptor swap dictionary')  
      acceptor_swap_dict=BuildCHARMMHBondAcceptorEquivalenceDict()
  [donor_list,acceptor_list]=GetHbondDonorAcceptorList(eh,hbond_donor_acceptor_dict,verbose)
  hb_list=[]
  hb_score=[]
  nframes=0
  for i in range(0,t.GetFrameCount(),stride):
    t.CopyFrame(i)
    nframes+=1
    hbond_list=GetHbondListFromDonorAcceptorLists(donor_list,acceptor_list)
    if swap:
      hbond_list=GetEquivalentHBonds(hbond_list,eh,swap,donor_swap_dict,acceptor_swap_dict,verbose)
      for hb in hbond_list:
        if hb in hbond_list[hbond_list.index(hb)+1:]:hbond_list.pop(hbond_list.index(hb))
    for hb in hbond_list:
      if hb in hb_list:hb_score[hb_list.index(hb)]+=1
      else:
        hb_score.append(1)
        hb_list.append(hb)
  hbond_list=[]
  hbond_score=[]
  for hb,s in zip(hb_list,hb_score):
    if s/float(nframes)>=cutoff:
      if swap:hbond_list.append(list(hb)[0])
      else:hbond_list.append(hb)
      hbond_score.append(s/float(nframes))
  return (hbond_list,hbond_score)

def GetHbondListBetweenViews(eh1,eh2,hbond_donor_acceptor_dict={},verbose=True):
  """
  return the list of hydrogen bonds formed between two Entity or EntityView.
  if no dictionary for the hbond donors and acceptors is specified
  it will use the standard CHARMM names to determine them
  """
  [donor_list1,acceptor_list1]=GetHbondDonorAcceptorList(eh1,hbond_donor_acceptor_dict,verbose)
  [donor_list2,acceptor_list2]=GetHbondDonorAcceptorList(eh2,hbond_donor_acceptor_dict,verbose)
  hbond_list=[]
  for donor in donor_list1:
    for acceptor in acceptor_list2:
      if AreHBonded(donor,acceptor):
        hb=HBond(donor,acceptor)
        if hb in hbond_list:continue
        hbond_list.append(hb)
  for donor in donor_list2:
    for acceptor in acceptor_list1:
      if AreHBonded(donor,acceptor):
        hb=HBond(donor,acceptor)
        if hb in hbond_list:continue
        hbond_list.append(hb)
  return hbond_list

def GetEquivalentHBonds(ref_hbond_list,eh,swap=False,donor_swap_dict={},acceptor_swap_dict={},verbose=True):
  hbond_list=[]
  if not swap:
    for hbond in ref_hbond_list:
      res1=eh.FindResidue(hbond.donor.heavy_atom.chain.name,hbond.donor.heavy_atom.residue.number.num)
      res2=eh.FindResidue(hbond.acceptor.atom.chain.name,hbond.acceptor.atom.residue.number.num)
      donor=HBondDonor.FromResidue(res1,hbond.donor.heavy_atom.name,hbond.donor.hydrogen.name,verbose)
      acceptor=HBondAcceptor.FromResidue(res2,hbond.acceptor.atom.name,[a.name for a in hbond.acceptor.antecedent_list],verbose)
      hbond_list.append(set([HBond(donor,acceptor)]))
  else:
    if not donor_swap_dict:
      print('use of standard CHARMM HBond donor swap dictionary')
      donor_swap_dict=BuildCHARMMHBondDonorEquivalenceDict()
    if not acceptor_swap_dict:
      print('use of standard CHARMM HBond acceptor swap dictionary')  
      acceptor_swap_dict=BuildCHARMMHBondAcceptorEquivalenceDict()
    for hbond in ref_hbond_list:
      res1=eh.FindResidue(hbond.donor.heavy_atom.chain.name,hbond.donor.heavy_atom.residue.number.num)
      res2=eh.FindResidue(hbond.acceptor.atom.chain.name,hbond.acceptor.atom.residue.number.num)
      donor=HBondDonor.FromResidue(res1,hbond.donor.heavy_atom.name,hbond.donor.hydrogen.name,verbose)
      acceptor=HBondAcceptor.FromResidue(res2,hbond.acceptor.atom.name,[a.name for a in hbond.acceptor.antecedent_list],verbose)
      donor_list=ListEquivalentDonors(donor,donor_swap_dict)
      acceptor_list=ListEquivalentAcceptors(acceptor,acceptor_swap_dict)
      hbond_list.append(set([HBond(d,a) for d in donor_list for a in acceptor_list]))
  return hbond_list
    
    
def CalculateHBondScore(ref_eh,eh2,ref_eh2=None,hbond_donor_acceptor_dict={},swap=False,donor_swap_dict={},acceptor_swap_dict={},verbose=True):
  """
  Returns the fraction of H-bonds from ref_eh that are also present in eh2.
  If ref_eh2 is specified, it uses as reference the Hbonds between ref_eh and ref_eh2. 
  This allows to look at H-bonds between specific parts of proteins or so.
  Alternatively ref_eh can be a list of H-bonds.
  This function relies on atom names to determine the list of H-bond donors and acceptors.
  These names are given in a dictionary, which defaults to CHARMM.
  If swap is set to True, a dictionary for equivalent donors and one for equivalent acceptors
  (defaults to CHARMM) is used to check for equivalent HBonds in eh2.
  If swap is set to True, if two equivalent hydrogen bonds are present in the reference entity 
  (for example both oxygens of ASP H-bonding the same atom), it suffices that on of these bonds is
  present in eh2 for both of them to be counted as present in eh2.
  """
  if ref_eh2:hbond_list1=GetHbondListBetweenViews(ref_eh,ref_eh2,hbond_donor_acceptor_dict,verbose)
  elif type(ref_eh)==list:hbond_list1=ref_eh
  else:hbond_list1=GetHbondListFromView(ref_eh,hbond_donor_acceptor_dict,verbose)
  nbonds=float(len(hbond_list1))
  if nbonds==0:
    print('No HBonds in reference view')
    return None
  hbond_list2=GetEquivalentHBonds(hbond_list1,eh2,swap,donor_swap_dict,acceptor_swap_dict,verbose)
  c=0
  for hl in hbond_list2:
    for hbond in hl:
      if HBond(hbond.donor,hbond.acceptor).IsFormed():
        c+=1
        break
  return c/float(len(hbond_list1))


def AnalyzeHBondScore(ref_eh,t,eh2,ref_eh2=None,hbond_donor_acceptor_dict={},swap=False,donor_swap_dict={},acceptor_swap_dict={},first=0,last=-1,stride=1,verbose=True):
  """
  Returns the same score as CalculateHBondScore, but for a trajectory.
  """
  if swap:
    if not donor_swap_dict:
      print('use of standard CHARMM HBond donor swap dictionary')
      donor_swap_dict=BuildCHARMMHBondDonorEquivalenceDict()
    if not acceptor_swap_dict:
      print('use of standard CHARMM HBond acceptor swap dictionary')  
      acceptor_swap_dict=BuildCHARMMHBondAcceptorEquivalenceDict()
  if ref_eh2:hbond_list1=GetHbondListBetweenViews(ref_eh,ref_eh2,hbond_donor_acceptor_dict,verbose)
  elif type(ref_eh)==list:hbond_list1=ref_eh
  else:hbond_list1=GetHbondListFromView(ref_eh,hbond_donor_acceptor_dict,verbose)
  nbonds=float(len(hbond_list1))
  if nbonds==0:
    print('No HBonds in reference view')
    return None
  print('number of hbonds in ref_eh:',nbonds)
  hbond_list2=GetEquivalentHBonds(hbond_list1,eh2,swap,donor_swap_dict,acceptor_swap_dict,verbose)
  if last==-1:last=t.GetFrameCount()
  score=FloatList()
  for f in range(first,last,stride):
    t.CopyFrame(f)
    c=0
    for hl in hbond_list2:
      for hbond in hl:
        if hbond.IsFormed():
          c+=1
          break
    score.append(c/nbonds)
  return score


def GetHBondListIntersection(ref_hbond_list,ref_eh,hbond_list,swap=False,donor_swap_dict={},acceptor_swap_dict={}):
  if swap:
    if not donor_swap_dict:
      print('use of standard CHARMM HBond donor swap dictionary')
      donor_swap_dict=BuildCHARMMHBondDonorEquivalenceDict()
    if not acceptor_swap_dict:
      print('use of standard CHARMM HBond acceptor swap dictionary')  
      acceptor_swap_dict=BuildCHARMMHBondAcceptorEquivalenceDict()
  hbond_list=GetEquivalentHBonds(hbond_list,ref_eh,swap,donor_swap_dict,acceptor_swap_dict)
  ref_hbond_list=GetEquivalentHBonds(ref_hbond_list,ref_eh,swap,donor_swap_dict,acceptor_swap_dict)
  out_hbond_list=[]
  for hb in ref_hbond_list:
    if hb in hbond_list:
      out_hbond_list.append(hb)
  return out_hbond_list

  
  
  
  
  



import sys
import unittest
from ost import geom, conop, io
from ost.conop import cleanup

class TestCleanUp(unittest.TestCase):

  def setUp(self):
    self.comp_lib = conop.GetDefaultLib()
    io.profiles['DEFAULT'].processor = conop.RuleBasedProcessor(self.comp_lib)
    self.ent = io.LoadPDB("sample_test_cleanup.pdb")
    self.ent_no_wat = io.LoadPDB("sample_nowater.pdb")
    self.ent_no_lig = io.LoadPDB("sample_noligands.pdb")

  def testStripWater(self):
    self.new_ent = cleanup.Cleanup(self.ent, strip_water=True, canonicalize=False, remove_ligands=False)
    self.assertEqual( self.new_ent.residue_count, self.ent_no_wat.residue_count )
    self.assertTrue( self.new_ent.residues[0].IsValid() )
    self.assertEqual( self.new_ent.residues[0].qualified_name, self.ent_no_wat.residues[0].qualified_name)
    self.assertTrue( self.new_ent.residues[1].IsValid() )
    self.assertEqual( self.new_ent.residues[1].qualified_name, self.ent_no_wat.residues[1].qualified_name)
    self.assertTrue( self.new_ent.residues[2].IsValid() )
    self.assertEqual( self.new_ent.residues[2].qualified_name, self.ent_no_wat.residues[2].qualified_name)
    self.assertTrue( self.new_ent.residues[3].IsValid() )
    self.assertEqual( self.new_ent.residues[3].qualified_name, self.ent_no_wat.residues[3].qualified_name)
    self.assertTrue( self.new_ent.residues[4].IsValid() )
    self.assertEqual( self.new_ent.residues[4].qualified_name, self.ent_no_wat.residues[4].qualified_name)
    self.assertTrue( self.new_ent.residues[5].IsValid() )
    self.assertEqual( self.new_ent.residues[5].qualified_name, self.ent_no_wat.residues[5].qualified_name)
    self.assertTrue( self.new_ent.residues[6].IsValid() )
    self.assertEqual( self.new_ent.residues[6].qualified_name, self.ent_no_wat.residues[6].qualified_name)
    self.assertTrue( self.new_ent.residues[7].IsValid() )
    self.assertEqual( self.new_ent.residues[7].qualified_name, self.ent_no_wat.residues[7].qualified_name)

  def testCanonicalize(self):
    self.new_ent = cleanup.Cleanup(self.ent, strip_water=False, canonicalize=True, remove_ligands=False)
    #standard residue must be the same
    self.gly = self.ent.residues[1]
    self.new_gly = self.new_ent.residues[1]
    self.assertTrue(self.new_gly.IsValid())
    self.assertTrue(self.new_gly.IsPeptideLinking())
    self.assertEqual(self.gly.atom_count, self.new_gly.atom_count)
    #TEMP del sidechain of incomplete residue and OXT if present
    self.new_cys = self.new_ent.residues[4]
    self.new_cys_atoms = set([atm.name for atm in self.new_cys.atoms])
    self.assertEqual( len(self.new_cys_atoms), 4, msg = repr(self.new_cys_atoms))
    self.assertTrue( "CB" in self.new_cys_atoms)
    self.assertTrue( "CA" in self.new_cys_atoms)
    self.assertTrue( "C" in self.new_cys_atoms)
    self.assertFalse( "OXT" in self.new_cys_atoms)
    self.assertTrue( "N" in self.new_cys_atoms)
    #test replacement of atoms
    self.mse = self.ent.residues[0]
#    self.assertTrue( self.mse.IsValid())
#    self.assertTrue( self.mse.IsPeptideLinking())
    self.sel = self.mse.FindAtom("SE")
#    self.assertTrue( self.sel.IsValid())
    self.met = self.new_ent.residues[0]
    self.assertTrue(self.met.IsValid())
    self.assertEqual(self.mse.atom_count, self.met.atom_count)
    self.assertEqual(self.met.name, "MET")
    self.assertEqual(self.met.one_letter_code, "M")
    self.assertTrue(self.met.IsPeptideLinking())
    self.sul = self.met.FindAtom("SD")
    self.assertTrue(self.sul.IsValid())
    self.assertTrue(geom.Equal(self.sul.pos,self.sel.pos), msg = "sul:%s sel:%s"%(str(self.sul.pos), str(self.sel.pos)) )
    self.assertEqual(self.sul.element, "S")
#    self.AssertTrue( sul.mass == conop.Conopology.Instance().GetDefaultAtomMass("S"))
#    self.AssertTrue( sul.radius == conop.Conopology.Instance().GetDefaultAtomRadius("S"))
    for atm in self.met.atoms:
      self.assertFalse( atm.is_hetatom)
    #test addition
    self.mly = self.ent.residues[2]
#    self.assertTrue( self.mly.IsValid())
#    self.assertTrue( self.mly.IsPeptideLinking())
    self.new_lys = self.new_ent.residues[2]
    self.assertTrue(self.new_lys.IsValid())
    self.assertTrue(self.new_lys.IsPeptideLinking())
    self.assertEqual(self.new_lys.name, "LYS")
    self.assertEqual(self.new_lys.one_letter_code, "K")
    self.new_lys_atoms = set([atm.name for atm in self.new_lys.atoms])
    self.canon_lys = self.comp_lib.FindCompound("LYS")
    self.canon_lys_atoms = set([atom.name for atom in self.canon_lys.atom_specs
                                             if atom.element != "H" and atom.element != "D"  and not atom.is_leaving ])
    self.assertEqual(self.canon_lys_atoms, self.new_lys_atoms)
    self.assertFalse(self.canon_lys_atoms - self.new_lys_atoms)
    self.assertFalse(self.new_lys_atoms - self.canon_lys_atoms) #test the reverse
    for atm in self.new_lys.atoms:
      self.assertFalse( atm.is_hetatom)
    #deletions
    self.dha = self.ent.residues[3]
#    self.assertTrue( self.dha.IsValid())
#    self.assertTrue( self.dha.IsPeptideLinking())
    self.new_ser = self.new_ent.residues[3]
    self.assertTrue(self.new_ser.IsValid())
    self.assertTrue(self.new_ser.IsPeptideLinking())
    self.assertEqual(self.new_ser.name, "SER")
    self.assertEqual(self.new_ser.one_letter_code, "S")
    self.new_ser_atoms = set([atm.name for atm in self.new_ser.atoms])
    self.canon_ser = self.comp_lib.FindCompound("SER")
    self.canon_ser_atoms = set([atom.name for atom in self.canon_ser.atom_specs
                                             if atom.element != "H" and atom.element != "D"  and not atom.is_leaving ])
    #TEMP
    self.assertEqual( len(self.new_ser_atoms), 5)
    self.assertTrue( "CB" in self.new_ser_atoms)
    self.assertTrue( "CA" in self.new_ser_atoms)
    self.assertTrue( "C" in self.new_ser_atoms)
    self.assertTrue( "O" in self.new_ser_atoms)
    self.assertTrue( "N" in self.new_ser_atoms)
    #AFTER TEMP
    #self.assertEqual( self.canon_ser_atoms, self.new_ser_atoms)
    #self.assertFalse(self.canon_ser_atoms - self.new_ser_atoms)
    #self.assertFalse(self.new_ser_atoms - self.canon_ser_atoms) #test the reverse
    for atm in self.new_ser.atoms:
      self.assertFalse( atm.is_hetatom)
    #test deletion of whole residue
    self.assertEqual(self.ent.residues[5].chem_class, "D_PEPTIDE_LINKING")
    self.assertNotEqual(self.new_ent.residues[5].name, "DAL")
    self.assertNotEqual(self.ent.residue_count, self.new_ent.residue_count)

  def testRemoveLigands(self):
    self.new_ent = cleanup.Cleanup(self.ent, strip_water=False, canonicalize=False, remove_ligands=True)
    self.assertEqual(self.new_ent.residue_count, self.ent_no_lig.residue_count )
    #MSE
    self.assertTrue(self.new_ent.residues[0].IsValid() )
    self.assertEqual(self.new_ent.residues[0].qualified_name, self.ent_no_lig.residues[0].qualified_name)
    self.assertTrue(self.new_ent.residues[0].IsPeptideLinking())
    self.assertTrue(self.new_ent.residues[0].atoms[0].is_hetatom)
    #GLY
    self.assertTrue(self.new_ent.residues[1].IsValid() )
    self.assertEqual(self.new_ent.residues[1].qualified_name, self.ent_no_lig.residues[1].qualified_name)
    self.assertTrue(self.new_ent.residues[1].IsPeptideLinking())
    self.assertFalse(self.new_ent.residues[1].atoms[0].is_hetatom)
    #MLY
    self.assertTrue(self.new_ent.residues[2].IsValid() )
    self.assertEqual(self.new_ent.residues[2].qualified_name, self.ent_no_lig.residues[2].qualified_name)
    self.assertTrue(self.new_ent.residues[2].IsPeptideLinking())
    self.assertTrue(self.new_ent.residues[2].atoms[0].is_hetatom)
    #DHA
    self.assertTrue(self.new_ent.residues[3].IsValid() )
    self.assertEqual(self.new_ent.residues[3].qualified_name, self.ent_no_lig.residues[3].qualified_name)
    self.assertTrue(self.new_ent.residues[3].IsPeptideLinking())
    self.assertTrue(self.new_ent.residues[3].atoms[0].is_hetatom)
    #CYS
    self.assertTrue(self.new_ent.residues[4].IsValid() )
    self.assertEqual(self.new_ent.residues[4].qualified_name, self.ent_no_lig.residues[4].qualified_name)
    self.assertTrue(self.new_ent.residues[4].IsPeptideLinking())
    self.assertFalse(self.new_ent.residues[4].atoms[0].is_hetatom)
    #DAL
    self.assertTrue(self.new_ent.residues[5].IsValid() )
    self.assertEqual(self.new_ent.residues[5].qualified_name, self.ent_no_lig.residues[5].qualified_name)
    self.assertTrue(self.new_ent.residues[5].IsPeptideLinking())
    self.assertTrue(self.new_ent.residues[5].atoms[0].is_hetatom)
    #HOH
    self.assertTrue(self.new_ent.residues[6].IsValid() )
    self.assertEqual(self.new_ent.residues[6].qualified_name, self.ent_no_lig.residues[6].qualified_name)
    self.assertFalse(self.new_ent.residues[6].IsPeptideLinking()) # here assertFalse instead of assertTrue
    self.assertTrue(self.new_ent.residues[6].atoms[0].is_hetatom)


if __name__ == "__main__":
  from ost import testutils
  if testutils.SetDefaultCompoundLib():
    testutils.RunTests()
  else:
    print('No compound library available. Ignoring test_cleanup.py tests.')



#------------------------------------------------------------------------------
# This file is part of the OpenStructure project <www.openstructure.org>
#
# Copyright (C) 2008-2020 by the OpenStructure authors
#
# This library is free software; you can redistribute it and/or modify it under
# the terms of the GNU Lesser General Public License as published by the Free
# Software Foundation; either version 3.0 of the License, or (at your option)
# any later version.
# This library is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
# FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public License for more
# details.
#
# You should have received a copy of the GNU Lesser General Public License
# along with this library; if not, write to the Free Software Foundation, Inc.,
# 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
#------------------------------------------------------------------------------
# -*- coding: utf-8 -*-

from ost import gui
from ost import gfx
from PyQt5 import QtCore, QtWidgets
try: 
  from ost import img
  _img_present=True
except ImportError:
  _img_present=False
  pass
from .scene_selection_helper import SelHelper

#Wireframe Options
class WireframeWidget(QtWidgets.QWidget):
  def __init__(self, parent=None):
    QtWidgets.QWidget.__init__(self, parent)
    
    #Title
    self.text_ = "Wireframe"
        
    #Defaults
    min_line_width = 0.01
    max_line_width = 20
    
    #Set Render Mode
    self.mode_ = gfx.RenderMode.SIMPLE
        
    #Create Ui elements
    self.aa_rendering_cb_ = QtWidgets.QCheckBox()
  
    self.radius_spinbox_ = QtWidgets.QDoubleSpinBox()
    self.radius_spinbox_.setRange(min_line_width, max_line_width)
    self.radius_spinbox_.setDecimals(2)
    self.radius_spinbox_.setSingleStep(0.1)

    simple_label = QtWidgets.QLabel("Wireframe Settings")
    font = simple_label.font()
    font.setBold(True)
    
    radius_label = QtWidgets.QLabel("Line Width")
    aa_label = QtWidgets.QLabel("AA-Lines")

    grid = QtWidgets.QGridLayout()
    grid.addWidget(simple_label,0,0,1,3)
    grid.addWidget(aa_label, 1, 0, 1, 3)
    grid.addWidget(self.aa_rendering_cb_, 1, 2, 1, 1)
    grid.addWidget(radius_label,2,0,1,3)
    grid.addWidget(self.radius_spinbox_,2,2,1,1)
    grid.setRowStretch(5,1)
    self.setLayout(grid)
    
    self.radius_spinbox_.valueChanged.connect(self.UpdateLineWidth)
    self.aa_rendering_cb_.stateChanged.connect(self.UpdateAA)  
    
    self.setMinimumSize(250,100)
    
  def UpdateAA(self, value):
    scene_selection = gui.SceneSelection.Instance()
    for i in range(0,scene_selection.GetActiveNodeCount()):
      node = scene_selection.GetActiveNode(i)
      node.SetAALines(value)
          
  def UpdateLineWidth(self, value):
    scene_selection = gui.SceneSelection.Instance()
    for i in range(0,scene_selection.GetActiveNodeCount()):
      node = scene_selection.GetActiveNode(i)
      node.SetLineWidth(value)

  def UpdateGui(self):
    scene_selection = gui.SceneSelection.Instance()
    node = scene_selection.GetActiveNode(0)
    self.radius_spinbox_.setValue(node.GetLineWidth())

  def Update(self):
    self.setEnabled(True)
    self.UpdateGui()
    if SelHelper().CheckNotFlags(SelHelper.HAS_IMG | SelHelper.IS_ONE_TYPE):
      self.setEnabled(False)          

  def GetText(self):
    return self.text_

  def GetRenderMode(self):
    return self.mode_



import sys
import unittest
from ost import *
from ost import settings
from ost.bindings import clustalw

class TestClustalWBindings(unittest.TestCase):
  
  def setUp(self):
    self.protein = io.LoadEntity("testfiles/testprotein.pdb").Select("cname=A")

    self.targetseq = io.LoadSequence("testfiles/test.fasta")
    self.targetseq.AttachView(self.protein)
    self.templseq = io.LoadSequence("testfiles/similar.fasta")
    self.multseq = io.LoadSequenceList("testfiles/multiple.fasta")
    self.pw_alignment = io.LoadAlignment("testfiles/pairwise_aln.fasta")
    self.nopgap_pw_alignment = io.LoadAlignment("testfiles/nopgap_pairwise_aln.fasta")
    self.mult_alignment = io.LoadAlignment("testfiles/multiple_aln.fasta")

    self.strseq1 = self.targetseq.GetGaplessString()
    self.strseq2 = self.templseq.GetGaplessString()

    self.seq1 = io.LoadSequence("testfiles/seq1.fasta")
    self.seq2 = io.LoadSequence("testfiles/seq2.fasta")
    self.seq1_seq2_alignment = io.LoadAlignment("testfiles/seq1_seq2_aln.fasta")
    self.seq1_seq2_alignment_options_changed = io.LoadAlignment("testfiles/seq1_seq2_aln_options_changed.fasta")

  def testPairwiseClustalW(self):
    aln=clustalw.ClustalW(self.targetseq, self.templseq)
    assert self.pw_alignment.ToString(80) == aln.ToString(80), \
           "Pairwise alignment differs from precomputed one"
  
  def testNoPGapPariwiseClustalW(self):
    aln=clustalw.ClustalW(self.targetseq, self.templseq, nopgap=True)
    assert self.nopgap_pw_alignment.ToString(80) == aln.ToString(80), \
           "NoPGap pairwise alignment differs from precomputed one"

  def testAttachedViewClustalW(self):
    aln=clustalw.ClustalW(self.targetseq, self.templseq)
    assert aln.FindSequence("testseq").HasAttachedView(), \
           "Aligned sequence doesn't have an attached view"
    
  def testMultipleClustalW(self):
    aln=clustalw.ClustalW(self.multseq)
    assert self.mult_alignment.ToString(80) == aln.ToString(80), \
           "Multiple alignment differs from precomputed one"

  def testStringClustalW(self):
    aln=clustalw.ClustalW(self.strseq1, self.strseq2)
    aln.SetSequenceName(0,self.targetseq.GetName())
    aln.SetSequenceName(1,self.templseq.GetName())
    assert self.pw_alignment.ToString(80) == aln.ToString(80), \
           "Pairwise alignment using two strings differs from precomputed one \n%s \n%s" \
           %(self.pw_alignment.ToString(80),aln.ToString(80))

  def testPairwiseClustalWChangedOptions(self):
    # five residues removed two positions before the end of seq2
    aln=clustalw.ClustalW(self.seq1,self.seq2)
    assert self.seq1_seq2_alignment.ToString(80) == aln.ToString(80), \
           "Pairwise alignment with default gap penalties differs from precomputed one"
    aln=clustalw.ClustalW(self.seq1,self.seq2,clustalw_option_string="-GAPOPEN=2 -GAPEXT=0")
    assert self.seq1_seq2_alignment_options_changed.ToString(80) == aln.ToString(80), \
           "Pairwise alignment with modified gap penalties differs from precomputed one"

  def testUniqueIdentifier(self):
    # common case
    seq1 = seq.CreateSequence('heelloo', 'AWESOME')
    seq2 = seq.CreateSequence('heelloo', 'AWESOME')
    self.assertRaises(ValueError, clustalw.ClustalW, seq1, seq2)
    # nasty case with spaces
    seq2 = seq.CreateSequence('heelloo dear', 'AWESOME')
    self.assertRaises(ValueError, clustalw.ClustalW, seq1, seq2)

if __name__ == "__main__":
  # test if clustalw package is available on system, otherwise ignore tests
  try:
    clustalw_path=settings.Locate(('clustalw', 'clustalw2'))
  except(settings.FileNotFound):
    print("Could not find clustalw executable: ignoring unit tests")
    sys.exit(0)
  from ost import testutils
  testutils.RunTests()



import unittest
from ost import mol, conop


class TestCompound(unittest.TestCase):
    def setUp(self):
        self.compound_lib=conop.GetDefaultLib()

    def testFindCompound(self):
        compound=self.compound_lib.FindCompound('***')
        self.assertEqual(compound, None)
        compound=self.compound_lib.FindCompound('ALA')
        self.assertNotEqual(compound, None)
        self.assertEqual(compound.id, 'ALA')
        self.assertEqual(compound.three_letter_code, 'ALA')
        self.assertEqual(compound.one_letter_code, 'A')
        self.assertTrue(compound.IsPeptideLinking())
        self.assertEqual(compound.dialect, 'PDB')
        self.assertEqual(compound.formula, 'C3 H7 N O2')
        self.assertEqual(compound.chem_class, mol.L_PEPTIDE_LINKING)
        self.assertEqual(compound.inchi,
                        "1S/C3H7NO2/c1-2(4)3(5)6/h2H,4H2,1H3,(H,5,6)/t2-/m0/s1")
        self.assertEqual(compound.inchi_key, "QNAYBMKLOCPYGJ-REOHCLBHSA-N")

     
if __name__=='__main__':
    from ost import testutils
    if testutils.SetDefaultCompoundLib():
        testutils.RunTests()
    else:
        print('No compound library available. Ignoring test_compound.py tests.')


"""
A bunch of algorithms operating on two views. 

Authors: Marco Biasini, Pascal Benkert
"""

from ost import io,mol

def _GetChain(view):
  if view.chain_count!=1:
    raise RuntimeError("chain count of view must be one")
  return view.chains[0]
  
def _GetChains(view_a, view_b):
  return _GetChain(view_a), _GetChain(view_b)


def _EmptyView(view):
  if isinstance(view, mol.EntityHandle):
    return view.CreateEmptyView()
  return view.handle.CreateEmptyView()


def PairResiduesByNum(view_a, view_b, 
                      view_add_flags=mol.ViewAddFlag.INCLUDE_ATOMS):
  """
  Pair residues by residue number.
  """
  not_supported="PairResiduesByNum has no support for unsorted chains"
  chain_a, chain_b=_GetChains(view_a, view_b)
  if not chain_a.InSequence() or not chain_b.InSequence():
    raise RuntimeError(not_supported)
  residues_a=chain_a.residues
  residues_b=chain_b.residues
  residues_a=iter(chain_a.residues)
  residues_b=iter(chain_b.residues)
  result_a=_EmptyView(view_a)
  result_b=_EmptyView(view_b)
  try:
    while True:
      r1=next(residues_a)
      r2=next(residues_b)
      while r1.number<r2.number:
        r1=next(residues_a)
      while r2.number<r1.number:
        r2=next(residues_b)
      assert r1.number==r2.number
      result_a.AddResidue(r1, view_add_flags)
      result_b.AddResidue(r2, view_add_flags)
  except StopIteration:
    pass
  return result_a, result_b
  
def RepresentativeAtoms(ent, chain=None, alpha_and_beta=False):
  """
  Returns a view with one or two representative atom per amino acid residue. 

  There are two basic modes, controlled by the alpha_and_beta parameter:

  When the parameter is false, for residues with a sidechain, the C-beta atom is 
  used, for residues without  sidechain, the C-alpha atom is used. Note that this 
  is different from using the selection

    (aname=CA and rname=GLY) or (aname=CB and rname!=GLY)

  When the alpha_and_beta parameter is true, both C-alpha and C-beta (if
  available) are added to the view.

  If chain is not equal to None, only atoms of the chain with that chain name
  will be added to the view.
  """
  e_view=ent.CreateEmptyView()
  if ent.IsValid():
    # all chains:
    if chain==None:
      for res in ent.residues:
        res=res.handle
        if res.IsPeptideLinking():
          atom =  res.FindAtom('CB')
          if atom.IsValid():
            e_view.AddAtom(atom)   

            #also add C-alpha for hybrid-potential:
            if alpha_and_beta == 1:
              atom =  res.FindAtom('CA')
              if atom.IsValid():
                e_view.AddAtom(atom)

          else: # Cbeta does not exist, Calpha?
            atom =  res.FindAtom('CA')
            if atom.IsValid():
              e_view.AddAtom(atom)

              # count C_alphas twice for hybrid-potential:
              if alpha_and_beta == 1:
                e_view.AddAtom(atom)

    elif chain != "" and ent.FindChain(chain).IsValid():
      for res in ent.FindChain(chain).GetResidueList():
        res=res.handle
        if res.IsPeptideLinking():
          atom =  res.FindAtom('CB')
          if atom.IsValid():
            e_view.AddAtom(atom)

            if alpha_and_beta == 1:
              atom =  res.FindAtom('CA')
              if atom.IsValid():
                e_view.AddAtom(atom)

          else: # Cbeta does not exist, Calpha?
            atom =  res.FindAtom('CA')
            if atom.IsValid():
              e_view.AddAtom(atom)

              # count C_alphas twice for glycine:
              if alpha_and_beta == 1:
                e_view.AddAtom(atom)
  return e_view



import sys

if len(sys.argv) < 2:
  print("USAGE: python scripts/bump-version.py OST_VERSION")
  print("-> OST_VERSION is MAJOR.MINOR.PATCH (e.g. 1.9.1)")
  print("-> assumption is that a git tag OST_VERSION will exist")
  sys.exit(1)

# split up version number
version_string = sys.argv[1]
version = version_string.split('.')
major, minor, patch = (int(version[0]), int(version[1]), int(version[2]))

# fix CMakeLists
lines = open("CMakeLists.txt").readlines()
for i, line in enumerate(lines):
  if line.startswith("set (OST_VERSION_MAJOR"):
    lines[i] = "set (OST_VERSION_MAJOR %d)\n" % major
  elif line.startswith("set (OST_VERSION_MINOR"):
    lines[i] = "set (OST_VERSION_MINOR %d)\n" % minor
  elif line.startswith("set (OST_VERSION_PATCH"):
    lines[i] = "set (OST_VERSION_PATCH %d)\n" % patch
open("CMakeLists.txt", "w").writelines(lines)

# fix CHANGELOG
lines = open("CHANGELOG.txt").readlines()
for i, line in enumerate(lines):
  if line.startswith("Changes in Release") and "X" in line.upper():
    lines[i] = "Changes in Release %s\n" % version_string
open("CHANGELOG.txt", "w").writelines(lines)

# fix Docker recipe
lines = open("docker/Dockerfile").readlines()
for i, line in enumerate(lines):
  if line.startswith("ARG OPENSTRUCTURE_VERSION"):
    lines[i] = 'ARG OPENSTRUCTURE_VERSION="%s"\n' % version_string
open("docker/Dockerfile", "w").writelines(lines)

# fix Singularity recipe
vfile = "singularity/Singularity"
lines = open(vfile).readlines()
for i, line in enumerate(lines):
  if line.startswith("From: registry.scicore.unibas.ch/schwede/openstructure:"):
    lines[i] = 'From: registry.scicore.unibas.ch/schwede/openstructure:'+\
      '%s\n' % version_string
    break
open(vfile, "w").writelines(lines)
